@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: array.c,v 1.144 2005/02/14 15:54:52 mast Exp $
+|| $Id: array.c,v 1.158 2004/04/06 15:37:54 nilsson Exp $
 */
 
 #include "global.h"
@@ -20,20 +20,19 @@
 #include "pike_memory.h"
 #include "gc.h"
 #include "main.h"
-#include "security.h"
+#include "pike_security.h"
 #include "stuff.h"
 #include "bignum.h"
 #include "cyclic.h"
 #include "multiset.h"
-#include "mapping.h"
 
-RCSID("$Id: array.c,v 1.144 2005/02/14 15:54:52 mast Exp $");
+RCSID("$Id: array.c,v 1.158 2004/04/06 15:37:54 nilsson Exp $");
 
 PMOD_EXPORT struct array empty_array=
 {
   PIKE_CONSTANT_MEMOBJ_INIT(1), /* Never free */
   &weak_empty_array,     /* Next */
-  &weak_shrink_empty_array, /* previous (circular) */
+  0,			 /* previous */
   0,                     /* Size = 0 */
   0,                     /* malloced Size = 0 */
   0,                     /* no types */
@@ -55,16 +54,25 @@ PMOD_EXPORT struct array weak_empty_array=
 PMOD_EXPORT struct array weak_shrink_empty_array=
 {
   PIKE_CONSTANT_MEMOBJ_INIT(1),
-  &empty_array, &weak_empty_array, 0, 0, 0, ARRAY_WEAK_FLAG|ARRAY_WEAK_SHRINK,
+  0, &weak_empty_array, 0, 0, 0, ARRAY_WEAK_FLAG|ARRAY_WEAK_SHRINK,
   weak_shrink_empty_array.real_item,
 #ifdef HAVE_UNION_INIT
   {{0, 0, {0}}}, /* Only to avoid warnings. */
 #endif
 };
 
-struct array *gc_internal_array = &empty_array;
+struct array *first_array = &empty_array;
+struct array *gc_internal_array = 0;
 static struct array *gc_mark_array_pos = 0;
 
+#ifdef TRACE_UNFINISHED_TYPE_FIELDS
+PMOD_EXPORT int accept_unfinished_type_fields = 0;
+PMOD_EXPORT void dont_accept_unfinished_type_fields (void *orig)
+{
+  accept_unfinished_type_fields = (int) orig;
+}
+#endif
+
 
 /* Allocate an array, this might be changed in the future to
  * allocate linked lists or something
@@ -82,13 +90,10 @@ PMOD_EXPORT struct array *low_allocate_array(ptrdiff_t size, ptrdiff_t extra_spa
     return &empty_array;
   }
 
-  if( (size+extra_space-1) >
-      (ULONG_MAX-sizeof(struct array))/sizeof(struct svalue) )
-    Pike_error("Too large array (memory size exceeds size of size_t)\n");
   v=(struct array *)malloc(sizeof(struct array)+
 			   (size+extra_space-1)*sizeof(struct svalue));
   if(!v)
-    Pike_error(msg_out_of_mem);
+    Pike_error("Couldn't allocate array, out of memory.\n");
 
   GC_ALLOC(v);
 
@@ -104,7 +109,7 @@ PMOD_EXPORT struct array *low_allocate_array(ptrdiff_t size, ptrdiff_t extra_spa
   v->item=v->real_item;
   v->size = DO_NOT_WARN((INT32)size);
   INIT_PIKE_MEMOBJ(v);
-  LINK_ARRAY(v);
+  DOUBLELINK (first_array, v);
 
   for(e=0;e<v->size;e++)
   {
@@ -121,7 +126,7 @@ PMOD_EXPORT struct array *low_allocate_array(ptrdiff_t size, ptrdiff_t extra_spa
  */
 static void array_free_no_free(struct array *v)
 {
-  UNLINK_ARRAY(v);
+  DOUBLEUNLINK (first_array, v);
 
   free((char *)v);
 
@@ -163,14 +168,11 @@ PMOD_EXPORT void do_free_array(struct array *a)
 
 PMOD_EXPORT struct array *array_set_flags(struct array *a, int flags)
 {
-  flags = (flags & ~ARRAY_CONSIDER_REALLOC) |
-    (a->flags & ARRAY_CONSIDER_REALLOC);
-
   if (a->size)
     a->flags = flags;
   else {
     free_array(a);
-    switch (flags & (ARRAY_WEAK_FLAG|ARRAY_WEAK_SHRINK)) {
+    switch (flags) {
       case 0:
 	add_ref(a = &empty_array); break;
       case ARRAY_WEAK_FLAG:
@@ -200,6 +202,55 @@ PMOD_EXPORT void array_index(struct svalue *s,struct array *v,INT32 index)
   free_array(v);
 }
 
+/* Is destructive on data if destructive is set and it only has one ref. */
+PMOD_EXPORT struct array *array_column (struct array *data, struct svalue *index,
+					int destructive)
+{
+  int e;
+  struct array *a;
+  TYPE_FIELD types = 0;
+
+  DECLARE_CYCLIC();
+
+  /* Optimization */
+  if(data->refs == 1 && destructive)
+  {
+    /* An array with one ref cannot possibly be cyclic */
+    struct svalue sval;
+    data->type_field = BIT_MIXED | BIT_UNFINISHED;
+    for(e=0;e<data->size;e++)
+    {
+      index_no_free(&sval, ITEM(data)+e, index);
+      types |= 1 << sval.type;
+      free_svalue(ITEM(data)+e);
+      move_svalue (ITEM(data) + e, &sval);
+    }
+    data->type_field = types;
+    add_ref (data);
+    return data;
+  }
+
+  if((a=(struct array *)BEGIN_CYCLIC(data,0)))
+  {
+    add_ref(a);
+  }else{
+    push_array(a=allocate_array(data->size));
+    SET_CYCLIC_RET(a);
+
+    for(e=0;e<a->size;e++) {
+      index_no_free(ITEM(a)+e, ITEM(data)+e, index);
+      types |= 1 << ITEM(a)[e].type;
+    }
+    a->type_field = types;
+
+    dmalloc_touch_svalue(Pike_sp-1);
+    Pike_sp--;
+  }
+  END_CYCLIC();
+
+  return a;
+}
+
 PMOD_EXPORT void simple_array_index_no_free(struct svalue *s,
 				struct array *a,struct svalue *ind)
 {
@@ -214,12 +265,12 @@ PMOD_EXPORT void simple_array_index_no_free(struct svalue *s,
 	tmp.u.array=a;
 	if (a->size) {
 	  index_error(0,0,0,&tmp,ind,
-		      "Index %d is out of array range "
-		      "%d..%d.\n",
+		      "Index %"PRINTPIKEINT"d is out of array range "
+		      "%"PRINTPTRDIFFT"d..%"PRINTPTRDIFFT"d.\n",
 		      p, -a->size, a->size-1);
 	} else {
 	  index_error(0,0,0,&tmp,ind,
-		      "Attempt to index the empty array with %d.\n", p);
+		      "Attempt to index the empty array with %"PRINTPIKEINT"d.\n", p);
 	}
       }
       array_index_no_free(s,a,i);
@@ -228,13 +279,10 @@ PMOD_EXPORT void simple_array_index_no_free(struct svalue *s,
 
     case T_STRING:
     {
-      check_stack(4);
-      ref_push_array(a);
-      assign_svalue_no_free(Pike_sp++,ind);
-      f_column(2);
-      s[0]=Pike_sp[-1];
-      Pike_sp--;
-      dmalloc_touch_svalue(Pike_sp);
+      /* Set the type afterwards to avoid a clobbered svalue in case
+       * array_column throws. */
+      s->u.array = array_column (a, ind, 0);
+      s->type = T_ARRAY;
       break;
     }
 	
@@ -270,11 +318,11 @@ PMOD_EXPORT void simple_set_index(struct array *a,struct svalue *ind,struct sval
       INT_TYPE i = p < 0 ? p + a->size : p;
       if(i<0 || i>=a->size) {
 	if (a->size) {
-	  Pike_error("Index %d is out of array range "
-		     "%d..%d.\n",
+	  Pike_error("Index %"PRINTPIKEINT"d is out of array range "
+		     "%"PRINTPTRDIFFT"d..%"PRINTPTRDIFFT"d.\n",
 		     p, -a->size, a->size-1);
 	} else {
-	  Pike_error("Attempt to index the empty array with %d.\n", p);
+	  Pike_error("Attempt to index the empty array with %"PRINTPIKEINT"d.\n", p);
 	}
       }
       array_set_index(a,i,s);
@@ -330,8 +378,7 @@ PMOD_EXPORT struct array *array_insert(struct array *v,struct svalue *s,INT32 in
   }else{
     struct array *ret;
 
-    ret = array_set_flags(allocate_array_no_init(v->size+1, (v->size >> 3) + 1),
-			  v->flags);
+    ret=allocate_array_no_init(v->size+1, (v->size >> 3) + 1);
     ret->type_field = v->type_field;
 
     MEMCPY(ITEM(ret), ITEM(v), sizeof(struct svalue) * index);
@@ -351,40 +398,6 @@ PMOD_EXPORT struct array *array_insert(struct array *v,struct svalue *s,INT32 in
   return v;
 }
 
-/*
- * Shrink an array destructively
- */
-PMOD_EXPORT struct array *array_shrink(struct array *v, ptrdiff_t size)
-{
-  struct array *a;
-
-#ifdef PIKE_DEBUG
-  if(v->refs>2) /* Odd, but has to be two */
-    Pike_fatal("Array shrink on array with many references.\n");
-
-  if(size > v->size)
-    Pike_fatal("Illegal argument to array_shrink.\n");
-#endif
-
-  if(!size || (size*4 < v->malloced_size + 4)) /* Should we realloc it? */
-  {
-    a = array_set_flags(allocate_array_no_init(size, 0), v->flags);
-    if (a->size) {
-      a->type_field = v->type_field;
-    }
-
-    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
-    MEMCPY(ITEM(a), ITEM(v), size*sizeof(struct svalue));
-    v->size=0;
-    free_array(v);
-    return a;
-  } else if (v->size != size) {
-    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
-    v->size=size;
-  }
-  return v;
-}
-
 /*
  * resize array, resize an array destructively
  */
@@ -409,18 +422,52 @@ PMOD_EXPORT struct array *resize_array(struct array *a, INT32 size)
       }
       a->type_field |= BIT_INT;
       return a;
-    } else {
+    }else{
       struct array *ret;
-      ret = array_set_flags(low_allocate_array(size, (size>>1) + 4),
-			    a->flags);
-      MEMCPY(ITEM(ret), ITEM(a), sizeof(struct svalue)*a->size);
+      ret=low_allocate_array(size, (size>>1) + 4);
+      MEMCPY(ITEM(ret),ITEM(a),sizeof(struct svalue)*a->size);
       ret->type_field = DO_NOT_WARN((TYPE_FIELD)(a->type_field | BIT_INT));
       a->size=0;
       free_array(a);
       return ret;
     }
-  } else {
-    return array_shrink(a, size);
+  }else{
+    /* We should shrink the array */
+    free_svalues(ITEM(a)+size, a->size - size, a->type_field);
+    a->size = size;
+    return a;
+  }
+}
+
+/*
+ * Shrink an array destructively
+ */
+PMOD_EXPORT struct array *array_shrink(struct array *v, ptrdiff_t size)
+{
+  struct array *a;
+
+#ifdef PIKE_DEBUG
+  if(v->refs>2) /* Odd, but has to be two */
+    Pike_fatal("Array shrink on array with many references.\n");
+
+  if(size > v->size)
+    Pike_fatal("Illegal argument to array_shrink.\n");
+#endif
+
+  if(size*2 < v->malloced_size + 4) /* Should we realloc it? */
+  {
+    a=allocate_array_no_init(size,0);
+    a->type_field = v->type_field;
+
+    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
+    MEMCPY(ITEM(a), ITEM(v), size*sizeof(struct svalue));
+    v->size=0;
+    free_array(v);
+    return a;
+  }else{
+    free_svalues(ITEM(v) + size, v->size - size, v->type_field);
+    v->size=size;
+    return v;
   }
 }
 
@@ -437,15 +484,10 @@ PMOD_EXPORT struct array *array_remove(struct array *v,INT32 index)
 #endif
 
   array_free_index(v, index);
-  if (v->size == 1) {
-    v->size = 0;
-    /* NOTE: The following uses the fact that array_set_flags()
-     *       will reallocate the array if it has zero size!
-     */
-    return array_set_flags(v, v->flags);
-  } else if(v->size*4 + 4 < v->malloced_size ) /* Should we realloc it? */
+  if(v->size!=1 &&
+     v->size*2 + 4 < v->malloced_size ) /* Should we realloc it? */
   {
-    a = array_set_flags(allocate_array_no_init(v->size-1, 0), v->flags);
+    a=allocate_array_no_init(v->size-1, 0);
     a->type_field = v->type_field;
 
     if(index>0)
@@ -457,7 +499,7 @@ PMOD_EXPORT struct array *array_remove(struct array *v,INT32 index)
     v->size=0;
     free_array(v);
     return a;
-  } else {
+  }else{
     if(v->size-index>1)
     {
       MEMMOVE((char *)(ITEM(v)+index),
@@ -514,7 +556,7 @@ PMOD_EXPORT ptrdiff_t array_search(struct array *v, struct svalue *s,
 }
 
 /*
- * Slice a piece of an array (nondestructively)
+ * Slice a pice of an array (nondestructively)
  * return an array consisting of v[start..end-1]
  */
 PMOD_EXPORT struct array *slice_array(struct array *v, ptrdiff_t start,
@@ -552,17 +594,15 @@ PMOD_EXPORT struct array *slice_array(struct array *v, ptrdiff_t start,
 #endif
 
   a=allocate_array_no_init(end-start,0);
-  if (end-start) {
-    a->type_field = v->type_field;
+  a->type_field = v->type_field;
 
-    assign_svalues_no_free(ITEM(a), ITEM(v)+start, end-start, v->type_field);
-  }
+  assign_svalues_no_free(ITEM(a), ITEM(v)+start, end-start, v->type_field);
 
   return a;
 }
 
 /*
- * Slice a piece of an array (nondestructively)
+ * Slice a pice of an array (nondestructively)
  * return an array consisting of v[start..end-1]
  */
 PMOD_EXPORT struct array *friendly_slice_array(struct array *v,
@@ -718,10 +758,87 @@ INT32 *get_order(struct array *v, cmpfun fun)
   return current_order;
 }
 
+/* Returns 2 if no relation is established through lfun calls. */
+static INLINE int lfun_cmp (const struct svalue *a, const struct svalue *b)
+{
+  int fun;
+
+  if (a->type == T_OBJECT && a->u.object->prog) {
+    if ((fun = FIND_LFUN(a->u.object->prog,LFUN_LT)) != -1) {
+      push_svalue(b);
+      apply_low (a->u.object, fun, 1);
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
+      {
+	pop_stack();
+	return -1;
+      }
+      pop_stack();
+    }
+
+    if ((fun = FIND_LFUN(a->u.object->prog,LFUN_GT)) != -1) {
+      push_svalue(b);
+      apply_low (a->u.object, fun, 1);
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
+      {
+	pop_stack();
+	return 1;
+      }
+      pop_stack();
+    }
+
+    if ((fun = FIND_LFUN(a->u.object->prog,LFUN_EQ)) != -1) {
+      push_svalue(b);
+      apply_low (a->u.object, fun, 1);
+      if (!UNSAFE_IS_ZERO(Pike_sp-1)) {
+	pop_stack();
+	return 0;
+      }
+      pop_stack();
+    }
+  }
+
+  if(b->type == T_OBJECT && b->u.object->prog) {
+    if ((fun = FIND_LFUN(b->u.object->prog,LFUN_LT)) != -1) {
+      push_svalue(a);
+      apply_low (b->u.object, fun, 1);
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
+      {
+	pop_stack();
+	return 1;
+      }
+      pop_stack();
+    }
+
+    if ((fun = FIND_LFUN(b->u.object->prog,LFUN_GT)) != -1) {
+      push_svalue(a);
+      apply_low (b->u.object, fun, 1);
+      if(!UNSAFE_IS_ZERO(Pike_sp-1))
+      {
+	pop_stack();
+	return -1;
+      }
+      pop_stack();
+    }
+
+    if ((fun = FIND_LFUN(b->u.object->prog,LFUN_EQ)) != -1) {
+      push_svalue(a);
+      apply_low (b->u.object, fun, 1);
+      if (!UNSAFE_IS_ZERO(Pike_sp-1)) {
+	pop_stack();
+	return 0;
+      }
+      pop_stack();
+    }
+  }
+
+  return 2;
+}
+
 INLINE int set_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 {
-  INT32 def;
-  if(a->type == b->type)
+  int res, typediff = a->type - b->type;
+
+  if (!typediff)
   {
     switch(a->type)
     {
@@ -747,95 +864,38 @@ INLINE int set_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 
       case T_OBJECT:
 	if(a->u.object == b->u.object) return 0;
-	if(a->u.object->prog == b->u.object->prog) {
-	  if (a->u.object->prog) {
-	    if(a->u.object < b->u.object) {
-	      def = -1;
-	    } else {
-	      def = 1;
-	    }
-	  } else {
-	    /* Destructed objects are considered equal. */
-	    return 0;
-	  }
-	} else {
-	  /* Attempt to group objects cloned from the same program */
-	  if (a->u.object->prog < b->u.object->prog) {
-	    def = -1;
-	  } else {
-	    def = 1;
-	  }
-	}
 	break;
     }
-  }else{
-    def=a->type - b->type;
   }
 
-  if (a->type == T_OBJECT && a->u.object->prog) {
-    if (FIND_LFUN(a->u.object->prog,LFUN_LT) != -1) {
-      push_svalue(b);
-      apply_lfun(a->u.object,LFUN_LT,1);
-      if(!UNSAFE_IS_ZERO(Pike_sp-1))
-      {
-	pop_stack();
-	return -1;
-      }
-      pop_stack();
-    }
-    if (FIND_LFUN(a->u.object->prog,LFUN_GT) != -1) {
-      push_svalue(b);
-      apply_lfun(a->u.object,LFUN_GT,1);
-      if(!UNSAFE_IS_ZERO(Pike_sp-1))
-      {
-	pop_stack();
-	return 1;
-      }
-      pop_stack();
-    }
-    if (FIND_LFUN(a->u.object->prog,LFUN_EQ) != -1) {
-      push_svalue(b);
-      apply_lfun(a->u.object,LFUN_EQ,1);
-      if (!UNSAFE_IS_ZERO(Pike_sp-1)) {
-	pop_stack();
-	return 0;
-      }
-      pop_stack();
-    }
-  }
-  if(b->type == T_OBJECT && b->u.object->prog) {
-    if (FIND_LFUN(b->u.object->prog,LFUN_LT) != -1) {
-      push_svalue(a);
-      apply_lfun(b->u.object,LFUN_LT,1);
-      if(!UNSAFE_IS_ZERO(Pike_sp-1))
-      {
-	pop_stack();
-	return 1;
-      }
-      pop_stack();
-    }
-    if (FIND_LFUN(b->u.object->prog,LFUN_GT) != -1) {
-      push_svalue(a);
-      apply_lfun(b->u.object,LFUN_GT,1);
-      if(!UNSAFE_IS_ZERO(Pike_sp-1))
-      {
-	pop_stack();
-	return -1;
-      }
-      pop_stack();
-    }
-    if (FIND_LFUN(b->u.object->prog,LFUN_EQ) != -1) {
-      push_svalue(a);
-      apply_lfun(b->u.object,LFUN_EQ,1);
-      if (!UNSAFE_IS_ZERO(Pike_sp-1)) {
-	pop_stack();
-	return 0;
-      }
-      pop_stack();
-    }
-  }
+  res = lfun_cmp (a, b);
+  if (res != 2) return res;
 
-  return def;
+  if (typediff) return typediff;
+
+#ifdef PIKE_DEBUG
+  if (a->type != T_OBJECT)
+    Pike_fatal ("Expected objects when both types are the same.\n");
+#endif
+  if(a->u.object->prog == b->u.object->prog) {
+    if (a->u.object->prog) {
+      if(a->u.object < b->u.object) {
+	return -1;
+      } else {
+	return 1;
+      }
+    } else {
+      /* Destructed objects are considered equal. */
+      return 0;
+    }
+  } else {
+    /* Attempt to group objects cloned from the same program */
+    if (a->u.object->prog < b->u.object->prog) {
+      return -1;
+    } else {
+      return 1;
+    }
+  }
 }
 
 static int switch_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
@@ -867,7 +927,9 @@ static int switch_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 
 static int alpha_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 {
-  if (a->type == b->type)
+  int res, typediff = a->type - b->type;
+
+  if (!typediff)
   {
     switch(a->type)
     {
@@ -882,25 +944,62 @@ static int alpha_svalue_cmpfun(const struct svalue *a, const struct svalue *b)
 	return 0;
 	
       case T_STRING:
-	return DO_NOT_WARN((int)my_strcmp(a->u.string, b->u.string));
+	return DO_NOT_WARN((int)my_quick_strcmp(a->u.string, b->u.string));
 	
       case T_ARRAY:
 	if(a==b) return 0;
-	if(!a->u.array->size) return -1;
-	if(!b->u.array->size) return  1;
+	if (!a->u.array->size)
+	  if (!b->u.array->size) /* There are several different empty arrays. */
+	    return 0;
+	  else
+	    return -1;
+	else
+	  if (!b->u.array->size)
+	    return 1;
 	return alpha_svalue_cmpfun(ITEM(a->u.array), ITEM(b->u.array));
+
+      case T_MULTISET:
+	if (a == b) return 0;
+	{
+	  ptrdiff_t a_pos = multiset_first (a->u.multiset);
+	  ptrdiff_t b_pos = multiset_first (b->u.multiset);
+	  struct svalue ind_a, ind_b;
+	  if (a_pos < 0)
+	    if (b_pos < 0)
+	      return 0;
+	    else
+	      return -1;
+	  else
+	    if (b_pos < 0)
+	      return 1;
+	  res = alpha_svalue_cmpfun (
+	    use_multiset_index (a->u.multiset, a_pos, ind_a),
+	    use_multiset_index (b->u.multiset, b_pos, ind_b));
+	  sub_msnode_ref (a->u.multiset);
+	  sub_msnode_ref (b->u.multiset);
+	  return res;
+	}
+
+      case T_OBJECT:
+	if(a->u.object == b->u.object) return 0;
+	break;
 	
       default:
-	return set_svalue_cmpfun(a,b);
-	
-      case T_OBJECT:
-	break;
+#if 1
+	/* I think it would be better to leave the order undefined in
+	 * these cases since the addresses aren't observable
+	 * properties in pike. /mast */
+	if(a->u.refs < b->u.refs) return -1;
+	if(a->u.refs > b->u.refs) return 1;
+#endif
+	return 0;
     }
-  }else{
-    if(a->type!=T_OBJECT && b->type!=T_OBJECT)
-      return a->type - b->type;
   }
-  return is_gt(a,b) ? 1 : is_gt(b,a) ? -1 : 0;
+
+  res = lfun_cmp (a, b);
+  if (res != 2) return res;
+
+  return typediff;
 }
 
 #define CMP(X,Y) alpha_svalue_cmpfun(X,Y)
@@ -918,6 +1017,47 @@ PMOD_EXPORT void sort_array_destructively(struct array *v)
   low_sort_svalues(ITEM(v), ITEM(v)+v->size-1);
 }
 
+#define SORT_BY_INDEX
+#define EXTRA_LOCALS int cmpfun_res;
+#define CMP(X,Y) ((cmpfun_res = alpha_svalue_cmpfun(svals + X, svals + Y)) ? \
+		  cmpfun_res : pos[X] - pos[Y])
+#define SWAP(X,Y) {							\
+  {struct svalue tmp = svals[X]; svals[X] = svals[Y]; svals[Y] = tmp;}	\
+  {int tmp = pos[X]; pos[X] = pos[Y]; pos[Y] = tmp;}			\
+}
+#define TYPE struct svalue
+#define ID low_stable_sort_svalues
+#define EXTRA_ARGS , struct svalue *svals, INT32 *pos, int size
+#define XARGS , svals, pos, size
+#include "fsort_template.h"
+#undef SORT_BY_INDEX
+#undef EXTRA_LOCALS
+#undef CMP
+#undef SWAP
+#undef TYPE
+#undef ID
+#undef EXTRA_ARGS
+#undef XARGS
+
+/* This sort is stable. The return value is like the one from
+ * get_alpha_order. */
+PMOD_EXPORT INT32 *stable_sort_array_destructively(struct array *v)
+{
+  INT32 *current_order;
+  ONERROR tmp;
+  int e;
+
+  if(!v->size) return NULL;
+
+  current_order=(INT32 *)xalloc(v->size * sizeof(INT32));
+  SET_ONERROR(tmp, free, current_order);
+  for(e=0; e<v->size; e++) current_order[e]=e;
+
+  low_stable_sort_svalues (0, v->size - 1, ITEM (v), current_order, v->size);
+
+  UNSET_ONERROR (tmp);
+  return current_order;
+}
 
 
 /*
@@ -1053,11 +1193,14 @@ PMOD_EXPORT void array_fix_type_field(struct array *v)
 
   if(v->flags & ARRAY_LVALUE)
   {
-    v->type_field=BIT_MIXED;
+    v->type_field=BIT_MIXED|BIT_UNFINISHED;
     return;
   }
 
-  for(e=0; e<v->size; e++) t |= 1 << ITEM(v)[e].type;
+  for(e=0; e<v->size; e++) {
+    check_svalue (ITEM(v) + e);
+    t |= 1 << ITEM(v)[e].type;
+  }
 
 #ifdef PIKE_DEBUG
   if(t & ~(v->type_field))
@@ -1082,6 +1225,13 @@ void array_check_type_field(struct array *v)
   if(v->flags & ARRAY_LVALUE)
     return;
 
+#ifdef TRACE_UNFINISHED_TYPE_FIELDS
+  if (v->type_field & BIT_UNFINISHED && !accept_unfinished_type_fields) {
+    fputs ("Array got an unfinished type field.\n", stderr);
+    describe_something (v, T_ARRAY, 2, 2, 0, NULL);
+  }
+#endif
+
   for(e=0; e<v->size; e++)
   {
     if(ITEM(v)[e].type > MAX_TYPE)
@@ -1098,15 +1248,13 @@ void array_check_type_field(struct array *v)
 }
 #endif /* PIKE_DEBUG */
 
-PMOD_EXPORT struct array *compact_array(struct array *v) { return v; }
-
 /*
  * Get a pointer to the 'union anything' specified IF it is of the specified
  * type. The 'union anything' may be changed, but not the type.
  */
 PMOD_EXPORT union anything *low_array_get_item_ptr(struct array *a,
-						   INT32 ind,
-						   TYPE_T t)
+				       INT32 ind,
+				       TYPE_T t)
 {
   if(ITEM(a)[ind].type == t) return & (ITEM(a)[ind].u);
   return 0;
@@ -1119,8 +1267,8 @@ PMOD_EXPORT union anything *low_array_get_item_ptr(struct array *a,
  * the index as an svalue.
  */
 PMOD_EXPORT union anything *array_get_item_ptr(struct array *a,
-					       struct svalue *ind,
-					       TYPE_T t)
+				   struct svalue *ind,
+				   TYPE_T t)
 {
   INT_TYPE i, p;
   if(ind->type != T_INT)
@@ -1130,11 +1278,11 @@ PMOD_EXPORT union anything *array_get_item_ptr(struct array *a,
   i = p < 0 ? p + a->size : p;
   if(i<0 || i>=a->size) {
     if (a->size) {
-      Pike_error("Index %d is out of array range "
-		 "%d..%d.\n",
+      Pike_error("Index %"PRINTPIKEINT"d is out of array range "
+		 "%"PRINTPTRDIFFT"d..%"PRINTPTRDIFFT"d.\n",
 		 p, -a->size, a->size-1);
     } else {
-      Pike_error("Attempt to index the empty array with %d.\n", p);
+      Pike_error("Attempt to index the empty array with %"PRINTPIKEINT"d.\n", p);
     }
   }
   return low_array_get_item_ptr(a,i,t);
@@ -1443,11 +1591,6 @@ PMOD_EXPORT struct array *merge_array_with_order(struct array *a,
 #undef ID
 
 
-/*
- * merge two arrays and retain their order, this is done by arranging them
- * into ordered sets, merging them as sets and then rearranging the zipper
- * before zipping the sets together. 
- */
 PMOD_EXPORT struct array *merge_array_without_order2(struct array *a, struct array *b,INT32 op)
 {
   ONERROR r1,r2,r3,r4,r5;
@@ -1739,14 +1882,21 @@ void describe_array_low(struct array *a, struct processing *p, int indent)
   }
 }
 
-PMOD_EXPORT void simple_describe_array(struct array *a)
+
+#ifdef PIKE_DEBUG
+void simple_describe_array(struct array *a)
 {
+  dynamic_buffer save_buf;
   char *s;
-  init_buf();
-  describe_array_low(a,0,0);
-  s=simple_free_buf();
-  fprintf(stderr,"({\n%s\n})\n",s);
-  free(s);
+  if (a->size) {
+    init_buf(&save_buf);
+    describe_array_low(a,0,0);
+    s=simple_free_buf(&save_buf);
+    fprintf(stderr,"({\n%s\n})\n",s);
+    free(s);
+  }
+  else
+    fputs ("({ })\n", stderr);
 }
 
 void describe_index(struct array *a,
@@ -1756,6 +1906,7 @@ void describe_index(struct array *a,
 {
   describe_svalue(ITEM(a)+e, indent, p);
 }
+#endif
 
 
 void describe_array(struct array *a,struct processing *p,int indent)
@@ -1867,7 +2018,9 @@ PMOD_EXPORT struct array *explode(struct pike_string *str,
       if(ret->size == ret->malloced_size)
       {
 	e=ret->size;
-	ret=resize_array(ret, e * 2);
+	ACCEPT_UNFINISHED_TYPE_FIELDS {
+	  ret=resize_array(ret, e * 2);
+	} END_ACCEPT_UNFINISHED_TYPE_FIELDS;
 	ret->size=e;
       }
 
@@ -1883,7 +2036,9 @@ PMOD_EXPORT struct array *explode(struct pike_string *str,
     if(ret->size == ret->malloced_size)
     {
       e=ret->size;
-      ret=resize_array(ret, e * 2);
+      ACCEPT_UNFINISHED_TYPE_FIELDS {
+	ret=resize_array(ret, e * 2);
+      } END_ACCEPT_UNFINISHED_TYPE_FIELDS;
       ret->size=e;
     }
 
@@ -1951,12 +2106,6 @@ PMOD_EXPORT struct array *copy_array_recursively(struct array *a,struct processi
   if(d_flag > 1)  array_check_type_field(a);
 #endif
 
-  if (!a->size) {
-    add_ref(&empty_array);
-    return array_set_flags(&empty_array,
-			   a->flags & ~(ARRAY_LVALUE|ARRAY_CONSIDER_REALLOC));
-  }
-
   doing.next=p;
   doing.pointer_a=(void *)a;
   for(;p;p=p->next)
@@ -1972,7 +2121,7 @@ PMOD_EXPORT struct array *copy_array_recursively(struct array *a,struct processi
   ret=allocate_array_no_init(a->size,0);
   doing.pointer_b=(void *)ret;
 
-  ret->flags = a->flags & ~(ARRAY_LVALUE|ARRAY_CONSIDER_REALLOC);
+  ret->flags = a->flags & ~ARRAY_LVALUE;
 
   copy_svalues_recursively_no_free(ITEM(ret),ITEM(a),a->size,&doing);
 
@@ -2060,8 +2209,17 @@ PMOD_EXPORT void check_array(struct array *a)
 {
   INT32 e;
 
-  if(a->next->prev != a)
-    Pike_fatal("Array check: a->next->prev != a\n");
+  if(a->next && a->next->prev != a)
+    Pike_fatal("array->next->prev != array.\n");
+
+  if(a->prev)
+  {
+    if(a->prev->next != a)
+      Pike_fatal("array->prev->next != array.\n");
+  }else{
+    if(first_array != a)
+      Pike_fatal("array->prev == 0 but first_array != array.\n");
+  }
 
   if(a->size > a->malloced_size)
     Pike_fatal("Array is larger than malloced block!\n");
@@ -2096,108 +2254,96 @@ PMOD_EXPORT void check_array(struct array *a)
 void check_all_arrays(void)
 {
   struct array *a;
-
-  a=&empty_array;
-  do
-  {
+  for (a = first_array; a; a = a->next)
     check_array(a);
-
-    a=a->next;
-    if(!a)
-      Pike_fatal("Null pointer in array list.\n");
-  } while (a != & empty_array);
 }
 #endif /* PIKE_DEBUG */
 
 
 static void gc_check_array(struct array *a)
 {
-  if(a->type_field & BIT_COMPLEX)
-  {
-    if (a->flags & ARRAY_WEAK_FLAG) {
-      debug_gc_check_weak_svalues(ITEM(a), a->size, T_ARRAY, a);
-      gc_checked_as_weak(a);
+  GC_ENTER (a, T_ARRAY) {
+    if(a->type_field & BIT_COMPLEX)
+    {
+      if (a->flags & ARRAY_WEAK_FLAG) {
+	gc_check_weak_svalues(ITEM(a), a->size);
+	gc_checked_as_weak(a);
+      }
+      else
+	gc_check_svalues(ITEM(a), a->size);
     }
-    else
-      debug_gc_check_svalues(ITEM(a), a->size, T_ARRAY, a);
-  }
+  } GC_LEAVE;
 }
 
 void gc_mark_array_as_referenced(struct array *a)
 {
-  if(gc_mark(a)) {
-#ifdef PIKE_DEBUG
-    if (a == &empty_array || a == &weak_empty_array || a == &weak_shrink_empty_array)
-      Pike_fatal("Trying to gc mark some *_empty_array.\n");
-#endif
-
-    if (a == gc_mark_array_pos)
-      gc_mark_array_pos = a->next;
-    if (a == gc_internal_array)
-      gc_internal_array = a->next;
-    else {
-      UNLINK_ARRAY(a);
-      LINK_ARRAY(a);		/* Linked in first. */
-    }
-
-    if (a->type_field & BIT_COMPLEX)
-    {
-      if (a->flags & ARRAY_WEAK_FLAG) {
-	int e;
-	TYPE_FIELD t;
-
-	if(a->flags & ARRAY_WEAK_SHRINK) {
-	  int d=0;
-#ifdef PIKE_DEBUG
-	  if (a->refs != 1)
-	    Pike_fatal("Got %d refs to weak shrink array "
-		       "which we'd like to change the size on.\n", a->refs);
-#endif
-	  t = 0;
-	  for(e=0;e<a->size;e++)
-	    if (!debug_gc_mark_weak_svalues(a->item+e, 1, T_ARRAY, a)) {
-	      a->item[d++]=a->item[e];
-	      t |= 1 << a->item[e].type;
-	    }
-	  if (d != a->size) {
-	    a->size=d;
-	    a->flags |= ARRAY_CONSIDER_REALLOC;
-	  }
-	}
-	else
-	  if (!(t = debug_gc_mark_weak_svalues(a->item, a->size, T_ARRAY, a)))
-	    t = a->type_field;
-
-	/* Ugly, but we are not allowed to change type_field
-	 * at the same time as the array is being built...
-	 * Actually we just need better primitives for building arrays.
-	 */
-	if(!(a->type_field & BIT_UNFINISHED) || a->refs!=1)
-	  a->type_field = t;
-	else
-	  a->type_field |= t;
-
-	gc_assert_checked_as_weak(a);
-      }
+  if(gc_mark(a))
+    GC_ENTER (a, T_ARRAY) {
+      if (a == gc_mark_array_pos)
+	gc_mark_array_pos = a->next;
+      if (a == gc_internal_array)
+	gc_internal_array = a->next;
       else {
-	TYPE_FIELD t;
-	if ((t = debug_gc_mark_svalues(ITEM(a), a->size, T_ARRAY, a))) {
+	DOUBLEUNLINK (first_array, a);
+	DOUBLELINK (first_array, a); /* Linked in first. */
+      }
+
+      if (a->type_field & BIT_COMPLEX)
+      {
+	if (a->flags & ARRAY_WEAK_FLAG) {
+	  int e;
+	  TYPE_FIELD t;
+
+	  if(a->flags & ARRAY_WEAK_SHRINK) {
+	    int d=0;
+#ifdef PIKE_DEBUG
+	    if (a->refs != 1)
+	      Pike_fatal("Got %d refs to weak shrink array "
+			 "which we'd like to change the size on.\n", a->refs);
+#endif
+	    t = 0;
+	    for(e=0;e<a->size;e++)
+	      if (!gc_mark_weak_svalues(a->item+e, 1)) {
+		a->item[d++]=a->item[e];
+		t |= 1 << a->item[e].type;
+	      }
+	    a->size=d;
+	  }
+	  else
+	    if (!(t = gc_mark_weak_svalues(a->item, a->size)))
+	      t = a->type_field;
+
+	  /* Ugly, but we are not allowed to change type_field
+	   * at the same time as the array is being built...
+	   * Actually we just need better primitives for building arrays.
+	   */
 	  if(!(a->type_field & BIT_UNFINISHED) || a->refs!=1)
 	    a->type_field = t;
 	  else
-	    a->type_field |= t;
+	    a->type_field |= t;	/* There might be an additional BIT_INT. */
+
+	  gc_assert_checked_as_weak(a);
+	}
+	else {
+	  TYPE_FIELD t;
+	  if ((t = gc_mark_svalues(ITEM(a), a->size))) {
+	    if(!(a->type_field & BIT_UNFINISHED) || a->refs!=1)
+	      a->type_field = t;
+	    else
+	      a->type_field |= t;
+	  }
+	  gc_assert_checked_as_nonweak(a);
 	}
-	gc_assert_checked_as_nonweak(a);
       }
-    }
-  }
+    } GC_LEAVE;
 }
 
 void real_gc_cycle_check_array(struct array *a, int weak)
 {
-  GC_CYCLE_ENTER(a, weak) {
+  GC_CYCLE_ENTER(a, T_ARRAY, weak) {
 #ifdef PIKE_DEBUG
-    if (a == &empty_array || a == &weak_empty_array || a == &weak_shrink_empty_array)
+    if (!gc_destruct_everything &&
+	(a == &empty_array || a == &weak_empty_array || a == &weak_shrink_empty_array))
       Pike_fatal("Trying to gc cycle check some *_empty_array.\n");
 #endif
 
@@ -2227,43 +2373,35 @@ void real_gc_cycle_check_array(struct array *a, int weak)
 unsigned gc_touch_all_arrays(void)
 {
   unsigned n = 0;
-  struct array *a = &empty_array;
-  do {
+  struct array *a;
+  if (!first_array || first_array->prev)
+    Pike_fatal ("error in array link list.\n");
+  for (a = first_array; a; a = a->next) {
     debug_gc_touch(a);
     n++;
-    if (!a->next || a->next->prev != a)
+    if (a->next && a->next->prev != a)
       Pike_fatal("Error in array link list.\n");
-    a=a->next;
-  } while (a != &empty_array);
+  }
   return n;
 }
 
 void gc_check_all_arrays(void)
 {
   struct array *a;
-  a=&empty_array;
-  do
-  {
+  for (a = first_array; a; a = a->next) {
 #ifdef PIKE_DEBUG
     if(d_flag > 1)  array_check_type_field(a);
 #endif
     gc_check_array(a);
-    a=a->next;
-  } while (a != & empty_array);
+  }
 }
 
 
 void gc_mark_all_arrays(void)
 {
   gc_mark_array_pos = gc_internal_array;
-  gc_mark(&empty_array);
-  gc_mark(&weak_empty_array);
-  gc_mark(&weak_shrink_empty_array);
-  while (gc_mark_array_pos != &empty_array) {
+  while (gc_mark_array_pos) {
     struct array *a = gc_mark_array_pos;
-#ifdef PIKE_DEBUG
-    if (!a) Pike_fatal("Null pointer in array list.\n");
-#endif
     gc_mark_array_pos = a->next;
     if(gc_is_referenced(a))
       gc_mark_array_as_referenced(a);
@@ -2273,7 +2411,7 @@ void gc_mark_all_arrays(void)
 void gc_cycle_check_all_arrays(void)
 {
   struct array *a;
-  for (a = gc_internal_array; a != &empty_array; a = a->next) {
+  for (a = gc_internal_array; a; a = a->next) {
     real_gc_cycle_check_array(a, 0);
     gc_cycle_run_queue();
   }
@@ -2281,7 +2419,7 @@ void gc_cycle_check_all_arrays(void)
 
 void gc_zap_ext_weak_refs_in_arrays(void)
 {
-  gc_mark_array_pos = empty_array.next;
+  gc_mark_array_pos = first_array;
   while (gc_mark_array_pos != gc_internal_array && gc_ext_weak_refs) {
     struct array *a = gc_mark_array_pos;
     gc_mark_array_pos = a->next;
@@ -2290,11 +2428,12 @@ void gc_zap_ext_weak_refs_in_arrays(void)
   gc_mark_discard_queue();
 }
 
-void gc_free_all_unreferenced_arrays(void)
+size_t gc_free_all_unreferenced_arrays(void)
 {
   struct array *a,*next;
+  size_t unreferenced = 0;
 
-  for (a = gc_internal_array; a != &weak_empty_array; a = next)
+  for (a = gc_internal_array; a; a = next)
   {
 #ifdef PIKE_DEBUG
     if (!a)
@@ -2313,7 +2452,10 @@ void gc_free_all_unreferenced_arrays(void)
     {
       next=a->next;
     }
+    unreferenced++;
   }
+
+  return unreferenced;
 }
 
 
@@ -2346,7 +2488,7 @@ void debug_dump_array(struct array *a)
 	  a == &weak_empty_array ? " (the weak_empty_array)" :
 	  a == &weak_shrink_empty_array ? " (the weak_shrink_empty_array)" :
 	  "");
-  fprintf(stderr,"Type field = ");
+  fprintf(stderr,"Type field =");
   debug_dump_type_field(a->type_field);
   fprintf(stderr,"\n");
   simple_describe_array(a);
@@ -2354,33 +2496,11 @@ void debug_dump_array(struct array *a)
 #endif
 
 
-void zap_all_arrays(void)
-{
-  struct array *a,*next;
-
-  for(a=empty_array.next;a!=&empty_array;a=next)
-  {
-
-#if defined(PIKE_DEBUG) && defined(DEBUG_MALLOC)
-    if(verbose_debug_exit && a!=&empty_array &&
-       a!=&weak_empty_array && a!=&weak_shrink_empty_array)
-      describe(a);
-#endif
-    
-    add_ref(a);
-    free_svalues(ITEM(a), a->size, a->type_field);
-    a->size=0;
-
-    SET_NEXT_AND_FREE(a,free_array);
-  }
-}
-
-
 void count_memory_in_arrays(INT32 *num_, INT32 *size_)
 {
   INT32 num=0, size=0;
   struct array *m;
-  for(m=empty_array.next;m!=&weak_empty_array;m=m->next)
+  for(m=first_array;m;m=m->next)
   {
     num++;
     size+=sizeof(struct array)+
