@@ -2,11 +2,11 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: main.c,v 1.163 2004/05/14 08:49:40 grubba Exp $
+|| $Id: main.c,v 1.200 2004/04/18 02:16:05 mast Exp $
 */
 
 #include "global.h"
-RCSID("$Id: main.c,v 1.163 2004/05/14 08:49:40 grubba Exp $");
+RCSID("$Id: main.c,v 1.200 2004/04/18 02:16:05 mast Exp $");
 #include "fdlib.h"
 #include "backend.h"
 #include "module.h"
@@ -31,12 +31,13 @@ RCSID("$Id: main.c,v 1.163 2004/05/14 08:49:40 grubba Exp $");
 #include "main.h"
 #include "operators.h"
 #include "rbtree.h"
-#include "security.h"
+#include "pike_security.h"
 #include "constants.h"
 #include "version.h"
 #include "program.h"
-#include "rusage.h"
+#include "pike_rusage.h"
 #include "module_support.h"
+#include "opcodes.h"
 
 #ifdef AUTO_BIGNUM
 #include "bignum.h"
@@ -48,7 +49,12 @@ RCSID("$Id: main.c,v 1.163 2004/05/14 08:49:40 grubba Exp $");
 
 #include "las.h"
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -61,7 +67,11 @@ RCSID("$Id: main.c,v 1.163 2004/05/14 08:49:40 grubba Exp $");
 #endif
 
 #ifdef TRY_USE_MMX
+#ifdef HAVE_MMX_H
 #include <mmx.h>
+#else
+#include <asm/mmx.h>
+#endif
 int try_use_mmx;
 #endif
 
@@ -81,7 +91,6 @@ PMOD_EXPORT int debug_options=0;
 PMOD_EXPORT int runtime_options=0;
 PMOD_EXPORT int d_flag=0;
 PMOD_EXPORT int c_flag=0;
-PMOD_EXPORT int t_flag=0;
 PMOD_EXPORT int default_t_flag=0;
 PMOD_EXPORT int a_flag=0;
 PMOD_EXPORT int l_flag=0;
@@ -185,6 +194,35 @@ static void get_master_key(HKEY cat)
 }
 #endif /* __NT__ */
 
+#ifdef __amigaos4__
+#define timeval timeval_amigaos
+#include <exec/types.h>
+#include <utility/hooks.h>
+#include <dos/dosextens.h>
+#include <proto/dos.h>
+
+static SAVEDS LONG scan_amigaos_environment_func(struct Hook *REG(a0,hook),
+						 APTR REG(a2,userdata),
+						 struct ScanVarsMsg *REG(a1,msg))
+{
+  if(msg->sv_GDir[0] == '\0' ||
+     !strcmp(msg->sv_GDir, "ENV:")) {
+    push_text(msg->sv_Name);
+    push_constant_text("=");
+    push_string(make_shared_binary_string(msg->sv_Var, msg->sv_VarLen));
+    f_add(3);
+  }
+
+  return 0;
+}
+
+static struct Hook scan_amigaos_environment_hook = {
+  { NULL, NULL },
+  (ULONG (*)())scan_amigaos_environment_func,
+  NULL, NULL
+};
+#endif /* __amigsos4__ */
+
 int dbm_main(int argc, char **argv)
 {
   JMP_BUF back;
@@ -197,6 +235,8 @@ int dbm_main(int argc, char **argv)
 
   TRACE((stderr, "dbm_main()\n"));
 
+  init_rusage();
+
   /* Attempt to make sure stderr is unbuffered. */
 #ifdef HAVE_SETVBUF
   setvbuf(stderr, NULL, _IONBF, 0);
@@ -246,9 +286,7 @@ int dbm_main(int argc, char **argv)
     /* This has already been done by initialize_dmalloc(). */
     init_callback_blocks();
 #endif /* !DEBUG_MALLOC */
-#ifdef PIKE_NEW_MULTISETS
     init_multiset();
-#endif
     init_builtin_constants();
   }
 
@@ -267,7 +305,6 @@ int dbm_main(int argc, char **argv)
   tzset();
 #endif /* HAVE_TZSET */
 
-#ifndef DONT_USE_SYSTEM_LOCALE
 #ifdef HAVE_SETLOCALE
 #ifdef LC_NUMERIC
   setlocale(LC_NUMERIC, "C");
@@ -285,7 +322,6 @@ int dbm_main(int argc, char **argv)
   setlocale(LC_MESSAGES, "");
 #endif
 #endif  
-#endif /* !DONT_USE_SYSTEM_LOCALE */
 
   TRACE((stderr, "Init master...\n"));
   
@@ -314,7 +350,7 @@ int dbm_main(int argc, char **argv)
   }
 
   TRACE((stderr, "Default master at \"%s\"...\n", master_file));
-  
+
   for(e=1; e<argc; e++)
   {
     TRACE((stderr, "Parse argument %d:\"%s\"...\n", e, argv[e]));
@@ -437,10 +473,12 @@ int dbm_main(int argc, char **argv)
 	      p++;
 	      goto more_d_flags;
 
+#ifdef DEBUG_MALLOC
 	    case 'g':
 	      debug_options|=GC_RESET_DMALLOC;
 	      p++;
 	      goto more_d_flags;
+#endif
 
 	    case 'p':
 	      debug_options|=NO_PEEP_OPTIMIZING;
@@ -486,11 +524,24 @@ int dbm_main(int argc, char **argv)
 	  break;
 
 	case 't':
-	  if(p[1]>='0' && p[1]<='9')
-	    t_flag+=STRTOL(p+1,&p,10);
-	  else
-	    t_flag++,p++;
-	  default_t_flag = t_flag;
+	  more_t_flags:
+	  switch (p[1]) {
+	    case '0': case '1': case '2': case '3': case '4':
+	    case '5': case '6': case '7': case '8': case '9':
+	      Pike_interpreter.trace_level+=STRTOL(p+1,&p,10);
+	      break;
+
+	    case 'g':
+	      gc_trace++;
+	      p++;
+	      goto more_t_flags;
+
+	    default:
+	      if (p[0] == 't')
+		Pike_interpreter.trace_level++;
+	      p++;
+	  }
+	  default_t_flag = Pike_interpreter.trace_level;
 	  break;
 
 	case 'p':
@@ -551,10 +602,10 @@ int dbm_main(int argc, char **argv)
    */
 #if STACK_DIRECTION < 0
   /* Equvivalent with |= 0xffff */
-  Pike_interpreter.stack_top += (~((size_t)Pike_interpreter.stack_top)) & 0xffff;
+  Pike_interpreter.stack_top += ~(PTR_TO_INT(Pike_interpreter.stack_top)) & 0xffff;
 #else /* STACK_DIRECTION >= 0 */
   /* Equvivalent with &= ~0xffff */
-  Pike_interpreter.stack_top -= ( ((size_t)Pike_interpreter.stack_top)) & 0xffff;
+  Pike_interpreter.stack_top -= PTR_TO_INT(Pike_interpreter.stack_top) & 0xffff;
 #endif /* STACK_DIRECTION < 0 */
 
 #ifdef PROFILING
@@ -679,27 +730,35 @@ int dbm_main(int argc, char **argv)
   TRACE((stderr, "Init time...\n"));
   
   GETTIMEOFDAY(&current_time);
-  
-  TRACE((stderr, "Init interpreter...\n"));
+
+  TRACE((stderr, "Init threads...\n"));
 
   low_th_init();
+
+  TRACE((stderr, "Init strings...\n"));
   
   init_shared_string_table();
+
+  TRACE((stderr, "Init interpreter...\n"));
+
   init_interpreter();
+
+  TRACE((stderr, "Init types...\n"));
+
   init_types();
-  init_lex();
+
+  TRACE((stderr, "Init opcodes...\n"));
+
+  init_opcodes();
+
+  TRACE((stderr, "Init programs...\n"));
+
   init_program();
+
+  TRACE((stderr, "Init objects...\n"));
+
   init_object();
 
-  TRACE((stderr, "Init modules...\n"));
-
-  init_modules();
-
-#ifdef TEST_MULTISET
-  /* A C-level testsuite for the low level stuff in multisets. */
-  test_multiset();
-#endif
-
   if(SETJMP(back))
   {
     if(throw_severity == THROW_EXIT)
@@ -717,7 +776,7 @@ int dbm_main(int argc, char **argv)
 	throw_value.type=T_INT;
 	err = (struct generic_error_struct *)
 	  get_storage (Pike_sp[-1].u.object, generic_error_program);
-	push_string (err->desc);
+	ref_push_string (err->error_message);
 	f_werror (1);
 	pop_stack();
       }
@@ -728,6 +787,15 @@ int dbm_main(int argc, char **argv)
   }else{
     back.severity=THROW_EXIT;
 
+    TRACE((stderr, "Init modules...\n"));
+
+    init_modules();
+
+#ifdef TEST_MULTISET
+    /* A C-level testsuite for the low level stuff in multisets. */
+    test_multiset();
+#endif
+
     TRACE((stderr, "Init master...\n"));
 
     master();
@@ -742,8 +810,21 @@ int dbm_main(int argc, char **argv)
       ITEM(a)[num].u.string=make_shared_string(argv[num]);
       ITEM(a)[num].type=T_STRING;
     }
+    a->type_field = BIT_STRING;
     push_array(a);
     
+#ifdef __amigaos__
+#ifdef __amigaos4__
+    if(DOSBase->dl_lib.lib_Version >= 50) {
+      struct svalue *mark = Pike_sp;
+      IDOS->ScanVars(&scan_amigaos_environment_hook,
+		     GVF_BINARY_VAR|GVF_DONT_NULL_TERM,
+		     NULL);
+      f_aggregate(Pike_sp-mark);
+    } else
+#endif
+      push_array(allocate_array_no_init(0,0));
+#else
     for(num=0;environ[num];num++);
     a=allocate_array_no_init(num,0);
     for(num=0;environ[num];num++)
@@ -751,7 +832,9 @@ int dbm_main(int argc, char **argv)
       ITEM(a)[num].u.string=make_shared_string(environ[num]);
       ITEM(a)[num].type=T_STRING;
     }
+    a->type_field = BIT_STRING;
     push_array(a);
+#endif
   
     apply(master(),"_main",2);
     pop_stack();
@@ -770,7 +853,7 @@ int dbm_main(int argc, char **argv)
 
 DECLSPEC(noreturn) void pike_do_exit(int num) ATTRIBUTE((noreturn))
 {
-  call_callback(&exit_callbacks, (void *)0);
+  call_callback(&exit_callbacks, NULL);
   free_callback_list(&exit_callbacks);
 
   exit_modules();
@@ -792,7 +875,7 @@ DECLSPEC(noreturn) void pike_do_exit(int num) ATTRIBUTE((noreturn))
 
 #ifdef PIKE_DEBUG
   /* For profiling */
-  exit_lex();
+  exit_opcodes();
 #endif
 
 #ifdef INTERNAL_PROFILING
@@ -831,7 +914,32 @@ void low_init_main(void)
 void exit_main(void)
 {
 #ifdef DO_PIKE_CLEANUP
-  cleanup_objects();
+  size_t count;
+
+#ifdef PIKE_DEBUG
+  if (verbose_debug_exit)
+    fprintf(stderr,"Exited normally, counting bytes.\n");
+#endif
+
+  /* Destruct all remaining objects while we have a proper execution
+   * environment. The downside is that the leak report below will
+   * always report destructed objects. We use the gc in a special mode
+   * for this to get a reasonably sane destruct order. */
+  gc_destruct_everything = 1;
+  count = do_gc (NULL, 1);
+  while (count) {
+    size_t new_count = do_gc (NULL, 1);
+    if (new_count >= count) {
+      fprintf (stderr, "Some destroy function is creating new objects "
+	       "during final cleanup - can't exit cleanly.\n");
+      break;
+    }
+    count = new_count;
+  }
+  gc_destruct_everything = 0;
+
+  /* Unload dynamic modules before static ones. */
+  exit_dynamic_load();
 #endif
 }
 
@@ -844,19 +952,18 @@ void low_exit_main(void)
 #ifdef DO_PIKE_CLEANUP
   void exit_iterators(void);
 
+  /* Clear various global references. */
+
 #ifdef AUTO_BIGNUM
   exit_auto_bignum();
 #endif
-
   exit_pike_searching();
-  th_cleanup();
   exit_object();
-  exit_dynamic_load();
   exit_signals();
   exit_builtin();
   exit_cpp();
   cleanup_interpret();
-  cleanup_added_efuns();
+  exit_builtin_constants();
   cleanup_module_support();
   exit_operators();
   exit_iterators();
@@ -864,6 +971,12 @@ void low_exit_main(void)
   cleanup_compiler();
   cleanup_error();
   exit_backend();
+  cleanup_gc();
+  cleanup_pike_types();
+
+  /* This zaps Pike_interpreter.thread_state among other things, so
+   * THREADS_ALLOW/DISALLOW are NOPs beyond this point. */
+  th_cleanup();
 
 #ifdef SHARED_NODES
   free(node_hash.table);
@@ -872,24 +985,12 @@ void low_exit_main(void)
   exit_pike_security();
   free_svalue(& throw_value);
   throw_value.type=T_INT;
-  {
-    while(1) {
-      int tmp=num_objects;
-      do_gc();
-      if(num_objects >= tmp) break;
-    }
-  }
 
-  cleanup_gc();
+  do_gc(NULL, 1);
 
-  cleanup_pike_types();
-
-#if defined(PIKE_DEBUG) && defined(DEBUG_MALLOC)
+#ifdef PIKE_DEBUG
   if(verbose_debug_exit)
   {
-    INT32 num,size,recount=0;
-    fprintf(stderr,"Exited normally, counting bytes.\n");
-
 #ifdef _REENTRANT
     if(count_pike_threads()>1)
     {
@@ -899,81 +1000,112 @@ void low_exit_main(void)
     }
 #endif
 
-
+#ifdef DEBUG_MALLOC
     search_all_memheaders_for_references();
+#endif
 
-    count_memory_in_arrays(&num, &size);
-    if(num)
+    /* The use of markers below only works after a gc run where it
+     * hasn't freed anything. Since we've destructed all objects in
+     * exit_main, nothing should be left after the run above, so only
+     * one more run is necessary. */
+    gc_keep_markers = 1;
+    do_gc (NULL, 1);
+
+#define STATIC_ARRAYS {&empty_array, &weak_empty_array, &weak_shrink_empty_array}
+
+#define REPORT_LINKED_LIST_LEAKS(TYPE, START, STATICS, T_TYPE, NAME) do { \
+      size_t num = 0;							\
+      struct TYPE *x;							\
+      for (x = START; x; x = x->next) {					\
+	struct marker *m = find_marker (x);				\
+	if (!m) {							\
+	  DO_IF_DEBUG (							\
+	    fprintf (stderr, "Didn't find gc marker as expected for:\n"); \
+	    describe_something (x, T_TYPE, 2, 2, 0, NULL);		\
+	  );								\
+	}								\
+	else {								\
+	  int is_static = 0;						\
+	  static const struct TYPE *statics[] = STATICS;		\
+	  ptrdiff_t i; /* Use signed type to avoid warnings from gcc. */ \
+	  for (i = 0; i < (ptrdiff_t) NELEM (statics); i++)		\
+	    if (x == statics[i])					\
+	      is_static = 1;						\
+	  if (x->refs != m->refs + is_static) {				\
+	    num++;							\
+	    fprintf (stderr, NAME " got %d unaccounted references:\n",	\
+		     x->refs - (m->refs + is_static));			\
+	    describe_something (x, T_TYPE, 2, 2, 0, NULL);		\
+	  }								\
+	}								\
+      }									\
+      if (num)								\
+	fprintf (stderr, NAME "s left: %"PRINTSIZET"d\n", num);		\
+    } while (0)
+
+    REPORT_LINKED_LIST_LEAKS (array, first_array, STATIC_ARRAYS, T_ARRAY, "Array");
+    REPORT_LINKED_LIST_LEAKS (multiset, first_multiset, {}, T_MULTISET, "Multiset");
+    REPORT_LINKED_LIST_LEAKS (mapping, first_mapping, {}, T_MAPPING, "Mapping");
+    REPORT_LINKED_LIST_LEAKS (program, first_program, {}, T_PROGRAM, "Program");
+    REPORT_LINKED_LIST_LEAKS (object, first_object, {}, T_OBJECT, "Object");
+
+#undef REPORT_LINKED_LIST_LEAKS
+
+    /* Just remove the extra external refs reported above and do
+     * another gc so that we don't report the blocks again in the low
+     * level dmalloc reports. */
+
+#define ZAP_LINKED_LIST_LEAKS(TYPE, START, STATICS) do {		\
+      struct TYPE *x;							\
+      for (x = START; x; x = x->next) {					\
+	struct marker *m = find_marker (x);				\
+	if (m) {							\
+	  int is_static = 0;						\
+	  static const struct TYPE *statics[] = STATICS;		\
+	  ptrdiff_t i; /* Use signed type to avoid warnings from gcc. */ \
+	  for (i = 0; i < (ptrdiff_t) NELEM (statics); i++)		\
+	    if (x == statics[i])					\
+	      is_static = 1;						\
+	  while (x->refs > m->refs + is_static) {			\
+	    m->flags |= GC_CLEANUP_FREED;				\
+	    PIKE_CONCAT(free_, TYPE) (x);				\
+	  }								\
+	}								\
+      }									\
+    } while (0)
+
+    ZAP_LINKED_LIST_LEAKS (array, first_array, STATIC_ARRAYS);
+    ZAP_LINKED_LIST_LEAKS (multiset, first_multiset, {});
+    ZAP_LINKED_LIST_LEAKS (mapping, first_mapping, {});
+    ZAP_LINKED_LIST_LEAKS (program, first_program, {});
+    ZAP_LINKED_LIST_LEAKS (object, first_object, {});
+
+#undef ZAP_LINKED_LIST_LEAKS
+
+    /* If we stumble on the real refs whose refcounts we've zapped
+     * above we should try to handle it gracefully. */
+    gc_external_refs_zapped = 1;
+
+    do_gc (NULL, 1);
+
+    gc_keep_markers = 0;
+    exit_gc();
+
+#ifdef DEBUG_MALLOC
     {
-      recount++;
-      fprintf(stderr,"Arrays left: %d (%d bytes) (zapped)\n",num,size);
+      INT32 num, size;
+      count_memory_in_pike_types(&num, &size);
+      if (num)
+	fprintf(stderr, "Types left: %d (%d bytes)\n", num, size);
+      describe_all_types();
     }
-
-    zap_all_arrays();
-
-    count_memory_in_mappings(&num, &size);
-    if(num)
-    {
-      recount++;
-      fprintf(stderr,"Mappings left: %d (%d bytes) (zapped)\n",num,size);
-    }
-
-    zap_all_mappings();
-
-    count_memory_in_multisets(&num, &size);
-    if(num)
-      fprintf(stderr,"Multisets left: %d (%d bytes)\n",num,size);
-
-
-    destruct_objects_to_destruct_cb();
-    if(recount)
-    {
-
-      fprintf(stderr,"Garbage collecting..\n");
-      do_gc();
-      
-      count_memory_in_arrays(&num, &size);
-      fprintf(stderr,"Arrays left: %d (%d bytes)\n",num,size);
-      count_memory_in_mappings(&num, &size);
-      fprintf(stderr,"Mappings left: %d (%d bytes)\n",num,size);
-      count_memory_in_multisets(&num, &size);
-      fprintf(stderr,"Multisets left: %d (%d bytes)\n",num,size);
-    }
-    
-
-    count_memory_in_programs(&num, &size);
-    if(num)
-      fprintf(stderr,"Programs left: %d (%d bytes)\n",num,size);
-
-    {
-      struct program *p;
-      for(p=first_program;p;p=p->next)
-	describe_something(p, T_PROGRAM, 0,2,0, NULL);
-    }
-
-
-    count_memory_in_objects(&num, &size);
-    if(num)
-      fprintf(stderr,"Objects left: %d (%d bytes)\n",num,size);
-
-    {
-      struct object *o;
-      for(o=first_object;o;o=o->next)
-	describe_something(o, T_OBJECT, 0,2,0, NULL);
-    }
-
-    count_memory_in_pike_types(&num, &size);
-    if (num)
-      fprintf(stderr, "Types left: %d (%d bytes)\n", num, size);
-    describe_all_types();
+#endif
   }
-#else
-
-  zap_all_arrays();
-  zap_all_mappings();
+#endif
 
   destruct_objects_to_destruct_cb();
-#endif
+
+  /* Now there are no arrays/objects/programs/anything left. */
 
   really_clean_up_interpret();
 
@@ -991,8 +1123,6 @@ void low_exit_main(void)
   free_all_object_blocks();
   first_program=0;
   free_all_program_blocks();
-#ifdef PIKE_NEW_MULTISETS
   exit_multiset();
 #endif
-#endif
 }
