@@ -1,9 +1,16 @@
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: charsetmod.c,v 1.41 2003/09/23 17:49:49 mast Exp $
+*/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include "../../global.h"
-RCSID("$Id: charsetmod.c,v 1.30 2003/09/23 17:59:24 mast Exp $");
+#include "global.h"
+RCSID("$Id: charsetmod.c,v 1.41 2003/09/23 17:49:49 mast Exp $");
 #include "program.h"
 #include "interpret.h"
 #include "stralloc.h"
@@ -13,8 +20,9 @@ RCSID("$Id: charsetmod.c,v 1.30 2003/09/23 17:59:24 mast Exp $");
 
 #include "iso2022.h"
 
-/* MUST BE LAST INCLUDE */
-#include "module_magic.h"
+
+#define sp Pike_sp
+#define fp Pike_fp
 
 #ifdef __CHAR_UNSIGNED__
 #define SIGNED signed
@@ -123,15 +131,16 @@ static int call_repcb(struct svalue *repcb, p_wchar2 ch)
   return 0;
 }
 
-#define REPLACE_CHAR(ch, func, ctx) \
+#define REPLACE_CHAR(ch, func, ctx, pos)	       \
           if(repcb != NULL && call_repcb(repcb, ch)) { \
 	    func(ctx, sb, sp[-1].u.string, rep, NULL); \
             pop_stack(); \
 	  } else if(rep != NULL) \
             func(ctx, sb, rep, NULL, NULL); \
 	  else \
-	    Pike_error("Character %lu unsupported by encoding.\n", \
-		       (unsigned long) ch);
+	    Pike_error("Character %lu at position %"PRINTPTRDIFFT"d "	\
+		       "unsupported by encoding.\n",			\
+		       (unsigned long) ch, (pos));
 
 #define MKREPCB(c) ((c).type == T_FUNCTION? &(c):NULL)
 
@@ -695,7 +704,7 @@ static void f_rfc1345(INT32 args)
 	case MODE_9696: lowtrans=32; lo=lo2=160; hi=hi2=255; break;
 	case MODE_BIG5: lowtrans=32; lo=0xa1; lo2=0x40; hi=0xf9; hi2=0xfe; break;
 	default:
-	  fatal("Internal error in rfc1345\n");
+	  Pike_fatal("Internal error in rfc1345\n");
 	}
 	
 	if(hi2) {
@@ -739,7 +748,7 @@ static void f_rfc1345(INT32 args)
       case MODE_9696: p = std_9696_program; break;
       case MODE_BIG5: p = std_big5_program; break;
       default:
-	fatal("Internal error in rfc1345\n");
+	Pike_fatal("Internal error in rfc1345\n");
       }
       push_object(clone_object(p, 0));
       ((struct std_rfc_stor *)(sp[-1].u.object->storage+std_rfc_stor_offs))
@@ -1000,11 +1009,11 @@ static void feed_utf8e(struct std_cs_stor *cs, struct string_builder *sb,
 	  string_builder_putchar(sb, 0x80|((c>>6)&0x3f));
 	  string_builder_putchar(sb, 0x80|(c&0x3f));	
 	} else
-	  REPLACE_CHAR(c, feed_utf8e, cs);
+	  REPLACE_CHAR(c, feed_utf8e, cs, p - STR2(str) - 1);
     }
     break;
   default:
-    fatal("Illegal shift size!\n");
+    Pike_fatal("Illegal shift size!\n");
   }
 }
 
@@ -1070,12 +1079,12 @@ static void feed_utf7_5e(struct std_cs_stor *cs, struct string_builder *sb,
 	  string_builder_putchar(sb, 0xc0|((c>>6)&0x3f));
 	  string_builder_putchar(sb, 0xc0|(c&0x3f));	
 	} else
-	  REPLACE_CHAR(c, feed_utf7_5e, cs);
+	  REPLACE_CHAR(c, feed_utf7_5e, cs, p - STR2(str) - 1);
       /* FIXME: Encode using surrogates? */
     }
     break;
   default:
-    fatal("Illegal shift size!\n");
+    Pike_fatal("Illegal shift size!\n");
   }
 }
 
@@ -1197,7 +1206,7 @@ static void feed_utf7e(struct utf7_stor *u7, struct string_builder *sb,
 	  u7->dat = dat;
 	  u7->shift = shift;
 	  u7->datbit = datbit;
-	  REPLACE_CHAR(c, feed_utf7e, u7);
+	  REPLACE_CHAR(c, feed_utf7e, u7, p - STR2(str) - 1);
 	  dat = u7->dat;
 	  shift = u7->shift;
 	  datbit = u7->datbit;
@@ -1229,7 +1238,7 @@ static void feed_utf7e(struct utf7_stor *u7, struct string_builder *sb,
     }
     break;
   default:
-    fatal("Illegal shift size!\n");
+    Pike_fatal("Illegal shift size!\n");
   }
 
   u7->dat = dat;
@@ -1308,7 +1317,7 @@ static void feed_std8e(struct std8e_stor *s8, struct string_builder *sb,
 	else if(c>=lo && c<hi && (ch=tab[c-lo])!=0)
 	  string_builder_putchar(sb, ch);
 	else
-	  REPLACE_CHAR(c, feed_std8e, s8)
+	  REPLACE_CHAR(c, feed_std8e, s8, p - STR0(str) - 1)
     }
     break;
   case 1:
@@ -1320,7 +1329,7 @@ static void feed_std8e(struct std8e_stor *s8, struct string_builder *sb,
 	else if(c>=lo && c<hi && (ch=tab[c-lo])!=0)
 	  string_builder_putchar(sb, ch);
 	else
-	  REPLACE_CHAR(c, feed_std8e, s8);
+	  REPLACE_CHAR(c, feed_std8e, s8, p - STR1(str) - 1);
     }
     break;
   case 2:
@@ -1332,11 +1341,11 @@ static void feed_std8e(struct std8e_stor *s8, struct string_builder *sb,
 	else if(c>=lo && c<hi && (ch=tab[c-lo])!=0)
 	  string_builder_putchar(sb, ch);
 	else
-	  REPLACE_CHAR(c, feed_std8e, s8);
+	  REPLACE_CHAR(c, feed_std8e, s8, p - STR2(str) - 1);
     }
     break;
   default:
-    fatal("Illegal shift size!\n");
+    Pike_fatal("Illegal shift size!\n");
   }
 }
 
@@ -1396,7 +1405,7 @@ static void feed_std16e(struct std16e_stor *s16, struct string_builder *sb,
 	    string_builder_putchar(sb, (ch>>8)&0xff);
 	  string_builder_putchar(sb, ch&0xff);
 	} else
-	  REPLACE_CHAR(c, feed_std16e, s16);
+	  REPLACE_CHAR(c, feed_std16e, s16, p - STR0(str) - 1);
     }
     break;
   case 1:
@@ -1410,7 +1419,7 @@ static void feed_std16e(struct std16e_stor *s16, struct string_builder *sb,
 	    string_builder_putchar(sb, (ch>>8)&0xff);
 	  string_builder_putchar(sb, ch&0xff);
 	} else
-	  REPLACE_CHAR(c, feed_std16e, s16);
+	  REPLACE_CHAR(c, feed_std16e, s16, p - STR1(str) - 1);
     }
     break;
   case 2:
@@ -1424,11 +1433,11 @@ static void feed_std16e(struct std16e_stor *s16, struct string_builder *sb,
 	    string_builder_putchar(sb, (ch>>8)&0xff);
 	  string_builder_putchar(sb, ch&0xff);
 	} else
-	  REPLACE_CHAR(c, feed_std16e, s16);
+	  REPLACE_CHAR(c, feed_std16e, s16, p - STR2(str) - 1);
     }
     break;
   default:
-    fatal("Illegal shift size!\n");
+    Pike_fatal("Illegal shift size!\n");
   }
 }
 
@@ -1448,11 +1457,11 @@ static void f_feed_std16e(INT32 args)
 }
 
 
-void pike_module_init(void)
+PIKE_MODULE_INIT
 {
   int i;
   struct svalue prog;
-  extern void iso2022_init();
+  extern void iso2022_init(void);
 
   iso2022_init();
 
@@ -1620,9 +1629,9 @@ void pike_module_init(void)
 			"function(string:string)|void:object)", 0);
 }
 
-void pike_module_exit(void)
+PIKE_MODULE_EXIT
 {
-  extern void iso2022_exit();
+  extern void iso2022_exit(void);
 
   if(utf7e_program != NULL)
     free_program(utf7e_program);
