@@ -1,14 +1,14 @@
-#line 1 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 1 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 /*
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: pcre_glue.cmod,v 1.16 2004/02/15 19:34:00 nilsson Exp $
+|| $Id: pcre_glue.cmod,v 1.17 2004/06/28 00:23:52 aldem Exp $
 || (original author: mirar)
 */
 
 #include "global.h"
-RCSID("$Id: pcre_glue.cmod,v 1.16 2004/02/15 19:34:00 nilsson Exp $");
+RCSID("$Id: pcre_glue.cmod,v 1.17 2004/06/28 00:23:52 aldem Exp $");
 #include "pcre_machine.h"
 
 #include "pike_macros.h"
@@ -76,12 +76,12 @@ struct cq__pcre_struct {
 #endif /* var_pattern_cq__pcre_defined */
 
 #ifdef var_re_cq__pcre_defined
-#line 45 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 45 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 pcre *re;
 #endif /* var_re_cq__pcre_defined */
 
 #ifdef var_extra_cq__pcre_defined
-#line 46 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 46 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 pcre_extra *extra;
 #endif /* var_extra_cq__pcre_defined */
 };
@@ -115,29 +115,29 @@ pcre_extra *extra;
    #define f_cq__pcre_create_defined
 ptrdiff_t f_cq__pcre_create_fun_num = 0;
 void f_cq__pcre_create(INT32 args) {
-#line 76 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 76 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 struct pike_string * pattern;
 struct svalue * options;
 struct svalue * table;
-#line 76 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 76 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args < 1) wrong_number_of_args_error("create",args,1);
-#line 76 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 76 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args > 3) wrong_number_of_args_error("create",args,3);
-#line 76 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 76 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[0-args].type != PIKE_T_STRING) SIMPLE_BAD_ARG_ERROR("create",1,"string");
-#line 76 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 76 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 debug_malloc_pass(pattern=Pike_sp[0-args].u.string);
 if (args > 1) {options=Pike_sp+1-args; dmalloc_touch_svalue(Pike_sp+1-args);
 } else options=0;
 if ((args > 2) && 
     ((Pike_sp[2-args].type != PIKE_T_INT) ||
      (Pike_sp[2-args].u.integer))) {
-#line 78 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 78 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[2-args].type != PIKE_T_OBJECT) SIMPLE_BAD_ARG_ERROR("create",3,"void|object");
-#line 78 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 78 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 table=Pike_sp+2-args; dmalloc_touch_svalue(Pike_sp+2-args);
 } else table=0;
-#line 79 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 79 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 {
 	 INT32 options=0;
 	 struct object *table=NULL;
@@ -193,7 +193,7 @@ table=Pike_sp+2-args; dmalloc_touch_svalue(Pike_sp+2-args);
    #define f_cq__pcre_study_defined
 ptrdiff_t f_cq__pcre_study_fun_num = 0;
 void f_cq__pcre_study(INT32 args) {
-#line 130 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 130 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args != 0) wrong_number_of_args_error("study",args,0);
 {
 	 const char *errmsg=NULL;
@@ -208,7 +208,7 @@ if(args != 0) wrong_number_of_args_error("study",args,0);
 	    Pike_error("error calling pcre_study: %s\n",errmsg);
 
 	 do { struct object * ret_=(this_object());  push_object(ret_); return; }while(0);
-#line 144 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 144 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 }
 
 }
@@ -217,20 +217,20 @@ if(args != 0) wrong_number_of_args_error("study",args,0);
    #define f_cq__pcre_cq__sprintf_defined
 ptrdiff_t f_cq__pcre_cq__sprintf_fun_num = 0;
 void f_cq__pcre_cq__sprintf(INT32 args) {
-#line 148 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 148 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 INT_TYPE c;
-#line 148 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 148 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 struct mapping * flags;
-#line 148 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 148 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args != 2) wrong_number_of_args_error("_sprintf",args,2);
-#line 148 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 148 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[0-2].type != PIKE_T_INT) SIMPLE_BAD_ARG_ERROR("_sprintf",1,"int");
 c=Pike_sp[0-2].u.integer;
-#line 148 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 148 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[1-2].type != PIKE_T_MAPPING) SIMPLE_BAD_ARG_ERROR("_sprintf",2,"mapping(mixed:mixed)");
-#line 148 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 148 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 debug_malloc_pass(flags=Pike_sp[1-2].u.mapping);
-#line 150 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 150 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 {
       switch(c)
       {
@@ -262,7 +262,7 @@ debug_malloc_pass(flags=Pike_sp[1-2].u.mapping);
    }
 
 }
-#line 180 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 180 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 #ifdef HAVE_PCRE_FULLINFO
 
    /*! @decl mapping info()
@@ -351,7 +351,7 @@ debug_malloc_pass(flags=Pike_sp[1-2].u.mapping);
    #define f_cq__pcre_info_defined
 ptrdiff_t f_cq__pcre_info_fun_num = 0;
 void f_cq__pcre_info(INT32 args) {
-#line 265 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 265 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args != 0) wrong_number_of_args_error("info",args,0);
 {
 	 int backrefmax,firstbyte,lastliteral,capturecount;
@@ -426,7 +426,7 @@ if(args != 0) wrong_number_of_args_error("info",args,0);
       }
 
 }
-#line 338 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 338 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 #endif /* HAVE_PCRE_FULLINFO */
 
 /*! @decl int|array exec(string subject,void|int startoffset)
@@ -496,20 +496,20 @@ if(args != 0) wrong_number_of_args_error("info",args,0);
    #define f_cq__pcre_exec_defined
 ptrdiff_t f_cq__pcre_exec_fun_num = 0;
 void f_cq__pcre_exec(INT32 args) {
-#line 404 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 404 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 struct pike_string * subject;
 struct svalue * startoffset;
-#line 404 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 404 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args < 1) wrong_number_of_args_error("exec",args,1);
-#line 404 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 404 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args > 2) wrong_number_of_args_error("exec",args,2);
-#line 404 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 404 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[0-args].type != PIKE_T_STRING) SIMPLE_BAD_ARG_ERROR("exec",1,"string");
-#line 404 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 404 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 debug_malloc_pass(subject=Pike_sp[0-args].u.string);
 if (args > 1) {startoffset=Pike_sp+1-args; dmalloc_touch_svalue(Pike_sp+1-args);
 } else startoffset=0;
-#line 406 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 406 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 {
 	 struct array *res;
 
@@ -522,11 +522,6 @@ if (args > 1) {startoffset=Pike_sp+1-args; dmalloc_touch_svalue(Pike_sp+1-args);
 	 if (!THIS->re)
 	    Pike_error("need to initialize before exec() is called\n");
 
-#ifdef PCRE_EXTRA_STUDY_DATA
-	 if (THIS->extra) opts|=PCRE_EXTRA_STUDY_DATA;
-#else
-	 /* FIXME: Throw an error if THIS->extra is set? */
-#endif /* PCRE_EXTRA_STUDY_DATA */
          /* FIXME: also MATCH_LIMIT and CALLOUT_DATA ... later */
 
 	 switch (args) /* get ints and check 8 bit compliancy */
@@ -564,27 +559,27 @@ if (args > 1) {startoffset=Pike_sp+1-args; dmalloc_touch_svalue(Pike_sp+1-args);
  *!    returns the number of a named subpattern
  */
 
-#line 459 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 454 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 #ifdef HAVE_PCRE_GET_STRINGNUMBER
    #define f_cq__pcre_get_stringnumber_defined
 ptrdiff_t f_cq__pcre_get_stringnumber_fun_num = 0;
 void f_cq__pcre_get_stringnumber(INT32 args) {
-#line 460 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 455 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 struct pike_string * stringname;
-#line 460 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 455 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args != 1) wrong_number_of_args_error("get_stringnumber",args,1);
-#line 460 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 455 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[0-1].type != PIKE_T_STRING) SIMPLE_BAD_ARG_ERROR("get_stringnumber",1,"string");
-#line 460 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 455 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 debug_malloc_pass(stringname=Pike_sp[0-1].u.string);
 {
 	 if (stringname->size_shift)
 	    SIMPLE_BAD_ARG_ERROR("get_stringnumber",1,"string (8bit)");
 	 do { INT_TYPE ret_=(pcre_get_stringnumber(THIS->re,stringname->str)); pop_stack(); push_int(ret_); return; }while(0);
-#line 465 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 460 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 }
 }
-#line 466 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 461 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 #endif /* HAVE_PCRE_GET_STRINGNUMBER */
 
 /* init and exit */
@@ -596,7 +591,7 @@ debug_malloc_pass(stringname=Pike_sp[0-1].u.string);
 #undef cq__pcre_event_handler_defined
 #define cq__pcre_event_handler_defined
 static void init_cq__pcre_struct(void)
-#line 471 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 466 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 {
 	 THIS->re=NULL;
 	 THIS->extra=NULL;
@@ -610,7 +605,7 @@ static void init_cq__pcre_struct(void)
 #undef cq__pcre_event_handler_defined
 #define cq__pcre_event_handler_defined
 static void exit_cq__pcre_struct(void)
-#line 478 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 473 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 {
 	 if (THIS->re) (*pcre_free)(THIS->re); /* -> free() usually */
 	 if (THIS->extra) (*pcre_free)(THIS->extra); /* -> free() usually */
@@ -638,7 +633,7 @@ static void cq__pcre_event_handler(int ev) {
 /*! @endclass
  */
 
-#line 488 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 483 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 #endif /* HAVE_LIBPCRE */
 
 /*! @decl array(string) split_subject(string subject, @
@@ -654,19 +649,19 @@ static void cq__pcre_event_handler(int ev) {
 #define f_split_subject_defined
 ptrdiff_t f_split_subject_fun_num = 0;
 void f_split_subject(INT32 args) {
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 struct pike_string * subject;
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 struct array * previous_result;
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(args != 2) wrong_number_of_args_error("split_subject",args,2);
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[0-2].type != PIKE_T_STRING) SIMPLE_BAD_ARG_ERROR("split_subject",1,"string");
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 debug_malloc_pass(subject=Pike_sp[0-2].u.string);
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 if(Pike_sp[1-2].type != PIKE_T_ARRAY) SIMPLE_BAD_ARG_ERROR("split_subject",2,"array(int)");
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 debug_malloc_pass(previous_result=Pike_sp[1-2].u.array);
 {
    int i,j;
@@ -690,11 +685,11 @@ debug_malloc_pass(previous_result=Pike_sp[1-2].u.array);
    }
 
    do { struct array * ret_=(res); pop_n_elems(2); push_array(ret_); return; }while(0);
-#line 523 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 518 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 }
 
 }
-#line 525 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 520 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 /*** module init & exit & stuff *****************************************/
 
 PIKE_MODULE_EXIT
@@ -707,7 +702,7 @@ PIKE_MODULE_EXIT
   }
 
 #endif /* class_cq__pcre_defined */
-#line 530 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 525 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 }
 
 PIKE_MODULE_INIT
@@ -1018,42 +1013,42 @@ PIKE_MODULE_INIT
 #endif /* cq__pcre_event_handler_defined */
 
 #ifdef f_cq__pcre_create_defined
-#line 76 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 76 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
   f_cq__pcre_create_fun_num =
     ADD_FUNCTION2("create", f_cq__pcre_create, tFunc(tString tOr(tVoid,"\10\200\0\0\0\177\377\377\377") tOr(tVoid,tObj),tVoid), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_cq__pcre_create_defined */
 
 #ifdef f_cq__pcre_study_defined
-#line 130 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 130 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
   f_cq__pcre_study_fun_num =
     ADD_FUNCTION2("study", f_cq__pcre_study, tFunc(tNone,tObj), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_cq__pcre_study_defined */
 
 #ifdef f_cq__pcre_cq__sprintf_defined
-#line 148 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 148 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
   f_cq__pcre_cq__sprintf_fun_num =
     ADD_FUNCTION2("_sprintf", f_cq__pcre_cq__sprintf, tFunc("\10\200\0\0\0\177\377\377\377" tMapping,tString), ID_STATIC, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_cq__pcre_cq__sprintf_defined */
 
 #ifdef f_cq__pcre_info_defined
-#line 265 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 265 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
   f_cq__pcre_info_fun_num =
     ADD_FUNCTION2("info", f_cq__pcre_info, tFunc(tNone,tMapping), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_cq__pcre_info_defined */
 
 #ifdef f_cq__pcre_exec_defined
-#line 404 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 404 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
   f_cq__pcre_exec_fun_num =
     ADD_FUNCTION2("exec", f_cq__pcre_exec, tFunc(tString tOr(tVoid,"\10\200\0\0\0\177\377\377\377"),tOr("\10\200\0\0\0\177\377\377\377",tArr("\10\200\0\0\0\177\377\377\377"))), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_cq__pcre_exec_defined */
 
 #ifdef f_cq__pcre_get_stringnumber_defined
-#line 460 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 455 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
   f_cq__pcre_get_stringnumber_fun_num =
     ADD_FUNCTION2("get_stringnumber", f_cq__pcre_get_stringnumber, tFunc(tString,"\10\200\0\0\0\177\377\377\377"), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
@@ -1064,12 +1059,12 @@ PIKE_MODULE_INIT
 #endif /* class_cq__pcre_defined */
 
 #ifdef f_split_subject_defined
-#line 500 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 495 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
   f_split_subject_fun_num =
     ADD_FUNCTION2("split_subject", f_split_subject, tFunc(tString tArr("\10\200\0\0\0\177\377\377\377"),tArr(tString)), 0, OPT_EXTERNAL_DEPEND|OPT_SIDE_EFFECT);
 
 #endif /* f_split_subject_defined */
-#line 808 "/tmp/pikedeb.ea910e171c/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
+#line 803 "/tmp/pikedeb.a8fa1cdb35/7.6/src/modules/_Regexp_PCRE/pcre_glue.cmod"
 #endif /* HAVE_LIBPCRE */
 }
 
