@@ -1,18 +1,74 @@
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
-
 /*
- * $Id: pike_memory.h,v 1.29 2000/12/13 21:31:53 hubbe Exp $
- */
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: pike_memory.h,v 1.49 2004/03/24 20:05:34 grubba Exp $
+*/
+
 #ifndef MEMORY_H
 #define MEMORY_H
 
 #include "global.h"
 #include "stralloc.h"
 
+#ifdef USE_VALGRIND
+
+#ifdef HAVE_MEMCHECK_H
+#include <memcheck.h>
+#elif defined(HAVE_VALGRIND_MEMCHECK_H)
+#include <valgrind/memcheck.h>
+#elif defined(HAVE_VALGRIND_H)
+#include <valgrind.h>
+#endif
+
+/* No Access */
+#define PIKE_MEM_NA(lvalue) do {					\
+    PIKE_MEM_NA_RANGE(&(lvalue), sizeof (lvalue));			\
+  } while (0)
+#define PIKE_MEM_NA_RANGE(addr, bytes) do {				\
+    VALGRIND_DISCARD(VALGRIND_MAKE_NOACCESS(addr, bytes));		\
+  } while (0)
+
+/* Write Only -- Will become RW when having been written to */
+#define PIKE_MEM_WO(lvalue) do {					\
+    PIKE_MEM_WO_RANGE(&(lvalue), sizeof (lvalue));			\
+  } while (0)
+#define PIKE_MEM_WO_RANGE(addr, bytes) do {				\
+    VALGRIND_DISCARD(VALGRIND_MAKE_WRITABLE(addr, bytes));		\
+  } while (0)
+
+/* Read/Write */
+#define PIKE_MEM_RW(lvalue) do {					\
+    PIKE_MEM_RW_RANGE(&(lvalue), sizeof (lvalue));			\
+  } while (0)
+#define PIKE_MEM_RW_RANGE(addr, bytes) do {				\
+    VALGRIND_DISCARD(VALGRIND_MAKE_READABLE(addr, bytes));		\
+  } while (0)
+
+/* Read Only -- Not currently supported by valgrind */
+#define PIKE_MEM_RO(lvalue) do {					\
+    PIKE_MEM_RO_RANGE(&(lvalue), sizeof (lvalue));			\
+  } while (0)
+#define PIKE_MEM_RO_RANGE(addr, bytes) do {				\
+    VALGRIND_DISCARD(VALGRIND_MAKE_READABLE(addr, bytes));		\
+  } while (0)
+
+/* Return true if a memchecker is in use. */
+#define PIKE_MEM_CHECKER() RUNNING_ON_VALGRIND
+
+#else
+
+#define PIKE_MEM_NA(lvalue)		do {} while (0)
+#define PIKE_MEM_NA_RANGE(addr, bytes)	do {} while (0)
+#define PIKE_MEM_WO(lvalue)		do {} while (0)
+#define PIKE_MEM_WO_RANGE(addr, bytes)	do {} while (0)
+#define PIKE_MEM_RW(lvalue)		do {} while (0)
+#define PIKE_MEM_RW_RANGE(addr, bytes)	do {} while (0)
+#define PIKE_MEM_RO(lvalue)		do {} while (0)
+#define PIKE_MEM_RO_RANGE(addr, bytes)	do {} while (0)
+#define PIKE_MEM_CHECKER()		0
+
+#endif /* USE_VALGRIND */
 
 
 #define MEMSEARCH_LINKS 512
@@ -43,72 +99,30 @@ struct mem_searcher
 };
 
 
-#if 1
-/* use new searching stuff */
-
 #include "pike_search.h"
 
-#else
-struct generic_mem_searcher
-{
-  char needle_shift;
-  char haystack_shift;
-  union data_u
-  {
-    struct mem_searcher eightbit;
-    struct other_search_s
-    {
-      enum methods method;
-      void *needle;
-      size_t needlelen;
-      int first_char;
-    } other;
-  } data;
-};
-
-#endif
-
 #include "block_alloc_h.h"
 #define MEMCHR0 MEMCHR
 
 /* Note to self: Prototypes must be updated manually /Hubbe */
 PMOD_EXPORT ptrdiff_t pcharp_memcmp(PCHARP a, PCHARP b, int sz);
 PMOD_EXPORT long pcharp_strlen(PCHARP a);
-PMOD_EXPORT INLINE p_wchar1 *MEMCHR1(p_wchar1 *p, p_wchar2 c, ptrdiff_t e);
-PMOD_EXPORT INLINE p_wchar2 *MEMCHR2(p_wchar2 *p, p_wchar2 c, ptrdiff_t e);
+PMOD_EXPORT p_wchar1 *MEMCHR1(p_wchar1 *p, p_wchar2 c, ptrdiff_t e);
+PMOD_EXPORT p_wchar2 *MEMCHR2(p_wchar2 *p, p_wchar2 c, ptrdiff_t e);
 PMOD_EXPORT void swap(char *a, char *b, size_t size);
 PMOD_EXPORT void reverse(char *memory, size_t nitems, size_t size);
 PMOD_EXPORT void reorder(char *memory, INT32 nitems, INT32 size,INT32 *order);
 PMOD_EXPORT size_t hashmem(const unsigned char *a, size_t len, size_t mlen);
 PMOD_EXPORT size_t hashstr(const unsigned char *str, ptrdiff_t maxn);
 PMOD_EXPORT size_t simple_hashmem(const unsigned char *str, ptrdiff_t len, ptrdiff_t maxn);
-PMOD_EXPORT void init_memsearch(struct mem_searcher *s,
-		    char *needle,
-		    size_t needlelen,
-		    size_t max_haystacklen);
-PMOD_EXPORT char *memory_search(struct mem_searcher *s,
-		    char *haystack,
-		    size_t haystacklen);
-PMOD_EXPORT void init_generic_memsearcher(struct generic_mem_searcher *s,
-			      void *needle,
-			      size_t needlelen,
-			      char needle_shift,
-			      size_t estimated_haystack,
-			      char haystack_shift);
-PMOD_EXPORT void *generic_memory_search(struct generic_mem_searcher *s,
-			    void *haystack,
-			    size_t haystacklen,
-			    char haystack_shift);
-PMOD_EXPORT char *my_memmem(char *needle,
-		size_t needlelen,
-		char *haystack,
-		size_t haystacklen);
+PMOD_EXPORT size_t simple_hashmem1(const p_wchar1 *str, ptrdiff_t len, ptrdiff_t maxn);
+PMOD_EXPORT size_t simple_hashmem2(const p_wchar2 *str, ptrdiff_t len, ptrdiff_t maxn);
 PMOD_EXPORT void memfill(char *to,
 	     INT32 tolen,
 	     char *from,
 	     INT32 fromlen,
 	     INT32 offset);
-PMOD_EXPORT char *debug_xalloc(size_t size);
+PMOD_EXPORT void *debug_xalloc(size_t size);
 PMOD_EXPORT void *debug_xmalloc(size_t s);
 PMOD_EXPORT void debug_xfree(void *mem);
 PMOD_EXPORT void *debug_xrealloc(void *m, size_t s);
