@@ -1,29 +1,48 @@
-/*\
-||| This file a part of Pike, and is copyright by Fredrik Hubinette
-||| Pike is distributed as GPL (General Public License)
-||| See the files COPYING and DISCLAIMER for more information.
-\*/
-/**/
+/*
+|| This file is part of Pike. For copyright information see COPYRIGHT.
+|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
+|| for more information.
+|| $Id: port.c,v 1.76 2003/10/24 23:38:56 nilsson Exp $
+*/
+
+/*
+ * This file defines things that may ought to already exist on the
+ * target system, but are missing in some ports. There are #ifdef's
+ * that will hopefully take care of everything.
+ */
+
+#define PORT_DO_WARN
+
 #include "global.h"
-#include "pike_macros.h"
 #include "time_stuff.h"
 #include "pike_error.h"
 
-#include <ctype.h>
-#include <math.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
+
+#include <ctype.h>
+#include <math.h>
 #include <errno.h>
 #include <float.h>
 #include <string.h>
 
-RCSID("$Id: port.c,v 1.37 2001/09/28 09:34:36 tomas Exp $");
+RCSID("$Id: port.c,v 1.76 2003/10/24 23:38:56 nilsson Exp $");
 
 #ifdef sun
 time_t time PROT((time_t *));
 #endif
 
+#if (SIZEOF_LONG == 4) && defined(_LP64)
+/* Kludge for gcc and the system header files not using the same model... */
+#undef LONG_MIN
+#undef LONG_MAX
+#undef ULONG_MAX
+#define LONG_MIN	INT_MIN
+#define LONG_MAX	INT_MAX
+#define ULONG_MAX	UINT_MAX
+#endif
+
 #ifndef HAVE_GETTIMEOFDAY
 
 #ifdef HAVE_GETSYSTEMTIMEASFILETIME
@@ -63,27 +82,17 @@ time_t TIME(time_t *t)
 }
 #endif
 
-#ifndef HUGE
-#define HUGE HUGE_VAL
-#endif /*!HUGE*/
+static unsigned INT32 RandSeed1 = 0x5c2582a4;
+static unsigned INT32 RandSeed2 = 0x64dff8ca;
 
-/*
- * This file defines things that may have to be changem when porting
- * LPmud to new environments. Hopefully, there are #ifdef's that will take
- * care of everything.
- */
-
-static unsigned long RandSeed1 = 0x5c2582a4;
-static unsigned long RandSeed2 = 0x64dff8ca;
-
-static unsigned long slow_rand(void)
+static unsigned INT32 slow_rand(void)
 {
   RandSeed1 = ((RandSeed1 * 13 + 1) ^ (RandSeed1 >> 9)) + RandSeed2;
   RandSeed2 = (RandSeed2 * RandSeed1 + 13) ^ (RandSeed2 >> 13);
   return RandSeed1;
 }
 
-static void slow_srand(long seed)
+static void slow_srand(INT32 seed)
 {
   RandSeed1 = (seed - 1) ^ 0xA5B96384UL;
   RandSeed2 = (seed + 1) ^ 0x56F04021UL;
@@ -93,22 +102,22 @@ static void slow_srand(long seed)
 #define RNDSTEP 7
 #define RNDJUMP 103
 
-static unsigned long rndbuf[ RNDBUF ];
+static unsigned INT32 rndbuf[ RNDBUF ];
 static int rnd_index;
 
-PMOD_EXPORT void my_srand(long seed)
+PMOD_EXPORT void my_srand(INT32 seed)
 {
   int e;
-  unsigned long mask;
+  unsigned INT32 mask;
 
   slow_srand(seed);
   
   rnd_index = 0;
   for (e=0;e < RNDBUF; e++) rndbuf[e]=slow_rand();
 
-  mask = (unsigned long) -1;
+  mask = (unsigned INT32) -1;
 
-  for (e=0;e< (int)sizeof(long)*8 ;e++)
+  for (e=0;e< (int)sizeof(INT32)*8 ;e++)
   {
     int d = RNDSTEP * e + 3;
     rndbuf[d % RNDBUF] &= mask;
@@ -117,24 +126,42 @@ PMOD_EXPORT void my_srand(long seed)
   }
 }
 
-PMOD_EXPORT unsigned long my_rand(void)
+PMOD_EXPORT unsigned INT32 my_rand(void)
 {
   if( ++rnd_index == RNDBUF) rnd_index=0;
   return rndbuf[rnd_index] += rndbuf[rnd_index+RNDJUMP-(rnd_index<RNDBUF-RNDJUMP?0:RNDBUF)];
 }
 
+#ifndef CONFIGURE_TEST
+
+PMOD_EXPORT void *pike_realloc(void *ptr, size_t sz)
+{
+  if (!ptr) return malloc(sz);
+#ifndef HAVE_WORKING_REALLOC_NULL
+#undef realloc
+#endif	/* !HAVE_WORKING_REALLOC_NULL */
+  return realloc(ptr, sz);
+#ifndef HAVE_WORKING_REALLOC_NULL
+#define realloc(PTR, SZ)	pike_realloc((PTR),(SZ))
+#endif	/* !HAVE_WORKING_REALLOC_NULL */
+}
+
+#endif	/* !CONFIGURE_TEST */
+
 #define DIGIT(x)	(isdigit(x) ? (x) - '0' : \
 			islower(x) ? (x) + 10 - 'a' : (x) + 10 - 'A')
 #define MBASE	('z' - 'a' + 1 + 10)
 
-long STRTOL(char *str,char **ptr,int base)
+long STRTOL(const char *str,char **ptr,int base)
 {
-  register long val;
-  register int c;
-  int xx, neg = 0;
+  /* Note: Code duplication in STRTOL_PCHARP and pcharp_to_svalue_inumber. */
 
-  if (ptr != (char **)0)
-    *ptr = str;			/* in case no number is formed */
+  unsigned long val, mul_limit;
+  int c;
+  int xx, neg = 0, add_limit, overflow = 0;
+
+  if (ptr != (char **)NULL)
+    *ptr = (char *)str;		/* in case no number is formed */
   if (base < 0 || base > MBASE)
     return (0);			/* base is invalid -- should be a fatal error */
   if (!isalnum(c = *str)) {
@@ -147,6 +174,7 @@ long STRTOL(char *str,char **ptr,int base)
       c = *++str;
     }
   }
+
   if (base == 0) {
     if (c != '0')
       base = 10;
@@ -155,21 +183,46 @@ long STRTOL(char *str,char **ptr,int base)
     else
       base = 8;
   }
+
   /*
    * for any base > 10, the digits incrementally following
    *	9 are assumed to be "abc...z" or "ABC...Z"
    */
   if (!isalnum(c) || (xx = DIGIT(c)) >= base)
     return (0);			/* no number formed */
-  if (base == 16 && c == '0' && isxdigit(((unsigned char *)str)[2]) &&
+  if (base == 16 && c == '0' && isxdigit(((const unsigned char *)str)[2]) &&
       (str[1] == 'x' || str[1] == 'X'))
     c = *(str += 2);		/* skip over leading "0x" or "0X" */
-  for (val = -DIGIT(c); isalnum(c = *++str) && (xx = DIGIT(c)) < base; )
-    /* accumulate neg avoids surprises near MAXLONG */
-    val = base * val - xx;
-  if (ptr != (char **)0)
-    *ptr = str;
-  return (neg ? val : -val);
+
+  mul_limit = LONG_MAX / base;
+  add_limit = (int) (LONG_MAX % base);
+  
+  if (neg) {
+    if (++add_limit == base) {
+      mul_limit++;
+      add_limit = 0;
+    }
+  }
+
+  for (val = DIGIT(c); isalnum(c = *++str) && (xx = DIGIT(c)) < base; ) {
+    if (val > mul_limit || (val == mul_limit && xx > add_limit))
+      overflow = 1;
+    else
+      val = base * val + xx;
+  }
+
+  if (ptr != (char **)NULL)
+    *ptr = (char *)str;
+  if (overflow) {
+    errno = ERANGE;
+    return neg ? LONG_MIN : LONG_MAX;
+  }
+  else {
+    if (neg)
+      return (long)(~val + 1);
+    else
+      return (long) val;
+  }
 }
 
 #ifndef HAVE_STRCASECMP
@@ -204,9 +257,9 @@ int STRNCMP(const char *a, const char *b, size_t maxlen)
 {
   size_t alen=STRNLEN(a,maxlen);
   size_t blen=STRNLEN(b,maxlen);
-  int ret=MEMCMP(a,b, a < b ? a : b);
+  int ret=MEMCMP(a,b, alen < blen ? alen : blen);
   if(ret) return ret;
-  return alen - blen;
+  return DO_NOT_WARN((int)(alen - blen));
 }
 #endif
 
@@ -219,9 +272,13 @@ void *MEMSET(void *s,int c,size_t n)
 }
 #endif
 
-#if defined(TRY_USE_MMX) || !defined(HAVE_MEMCPY) && !defined(HAVE_BCOPY)
+#if (0 && defined(TRY_USE_MMX)) || !defined(HAVE_MEMCPY) && !defined(HAVE_BCOPY)
 #ifdef TRY_USE_MMX
+#ifdef HAVE_MMX_H
 #include <mmx.h>
+#else
+#include <asm/mmx.h>
+#endif
 #endif
 PMOD_EXPORT void MEMCPY(void *bb,const void *aa,size_t s)
 {
@@ -325,19 +382,18 @@ PMOD_EXPORT void MEMMOVE(void *b,const void *aa,size_t s)
 #ifndef HAVE_MEMCMP
 PMOD_EXPORT int MEMCMP(const void *bb,const void *aa,size_t s)
 {
-  char *a=(char *)aa;
-  char *b=(char *)bb;
+  unsigned char *a=(unsigned char *)aa;
+  unsigned char *b=(unsigned char *)bb;
   for(;s;s--,b++,a++)
   {
     if(*b!=*a)
     {
       if(*b<*a) return -1;
-      if(*b>*a) return 1;
+      return 1;
     }
   }
   return 0;
 }
-
 #endif
 
 #ifndef HAVE_MEMCHR
@@ -345,13 +401,13 @@ PMOD_EXPORT void *MEMCHR(const void *p,char c,size_t e)
 {
   const char *t = p;
   while(e--) if(*(t++)==c) return t-1;
-  return (char *)0;
+  return (char *)NULL;
 }
 #endif
 
 
 #if !defined(HAVE_INDEX) && !defined(HAVE_STRCHR)
-PMOD_EXPORT char *STRCHR(char *s,char c)
+PMOD_EXPORT char *STRCHR(char *s,int c)
 {
   for(;*s;s++) if(*s==c) return s;
   return NULL;
@@ -406,9 +462,11 @@ PMOD_EXPORT char *STRTOK(char *s1,char *s2)
 
 /* Convert NPTR to a double.  If ENDPTR is not NULL, a pointer to the
    character after the last one used in the number is put in *ENDPTR.  */
-PMOD_EXPORT double STRTOD(char * nptr, char **endptr)
+PMOD_EXPORT double STRTOD(const char * nptr, char **endptr)
 {
-  register unsigned char *s;
+  /* Note: Code duplication in STRTOD_PCHARP. */
+
+  register const unsigned char *s;
   short int sign;
 
   /* The number so far.  */
@@ -426,7 +484,7 @@ PMOD_EXPORT double STRTOD(char * nptr, char **endptr)
     goto noconv;
   }
 
-  s = (unsigned char *)nptr;
+  s = (const unsigned char *)nptr;
 
   /* Eat whitespace.  */
   while (ISSPACE(*s)) ++s;
@@ -484,15 +542,21 @@ PMOD_EXPORT double STRTOD(char * nptr, char **endptr)
 
       errno = 0;
       ++s;
-      exp = STRTOL((char *)s, &end, 10);
+      exp = STRTOL((const char *)s, &end, 10);
       if (errno == ERANGE)
       {
 	/* The exponent overflowed a `long int'.  It is probably a safe
 	   assumption that an exponent that cannot be represented by
 	   a `long int' exceeds the limits of a `double'.  */
+	/* NOTE: Don't trust the value returned from STRTOL.
+	 * We need to find the sign of the exponent by hand.
+	 */
+	while(ISSPACE(*s)) {
+	  s++;
+	}
 	if (endptr != NULL)
 	  *endptr = end;
-	if (exp < 0)
+	if (*s == '-')
 	  goto underflow;
 	else
 	  goto overflow;
@@ -534,12 +598,14 @@ PMOD_EXPORT double STRTOD(char * nptr, char **endptr)
  overflow:
   /* Return an overflow error.  */
   errno = ERANGE;
-  return HUGE * sign;
+  return HUGE_VAL * sign;
 
  underflow:
   /* Return an underflow error.  */
+#if 0
   if (endptr != NULL)
     *endptr = (char *) nptr;
+#endif
   errno = ERANGE;
   return 0.0;
   
@@ -551,7 +617,7 @@ PMOD_EXPORT double STRTOD(char * nptr, char **endptr)
 }
 
 #ifndef HAVE_VSPRINTF
-PMOD_EXPORT int VSPRINTF(char *buf,char *fmt,va_list args)
+PMOD_EXPORT int VSPRINTF(char *buf,const char *fmt,va_list args)
 {
   char *b=buf;
   char *s;
@@ -581,7 +647,7 @@ PMOD_EXPORT int VSPRINTF(char *buf,char *fmt,va_list args)
       goto unknown_character;
 
     case 0:
-      fatal("Error in vsprintf format.\n");
+      Pike_fatal("Error in vsprintf format.\n");
       return 0;
 
     case '%':
@@ -609,7 +675,7 @@ PMOD_EXPORT int VSPRINTF(char *buf,char *fmt,va_list args)
     case 'E':
     case 'g':
       *fmt2p=0;
-      sprintf(buf,fmt2,va_arg(args,FLOAT_TYPE));
+      sprintf(buf,fmt2,va_arg(args,double));
       buf+=strlen(buf);
       break;
     }
@@ -622,15 +688,38 @@ PMOD_EXPORT int VSPRINTF(char *buf,char *fmt,va_list args)
 }
 #endif
 
+#ifndef HAVE_VSNPRINTF
+/* Warning: It's possible to trick this with something like
+ * snprintf("...%c...", 0). */
+PMOD_EXPORT int VSNPRINTF(char *buf, size_t size, const char *fmt, va_list args)
+{
+  int res;
+  buf[size - 1] = 0;
+  res = VSPRINTF (buf, fmt, args);
+  if (buf[size - 1]) Pike_fatal ("Buffer overflow in VSPRINTF.\n");
+  return res;
+}
+#endif
+
+#ifndef HAVE_SNPRINTF
+/* Warning: It's possible to trick this with something like
+ * snprintf("...%c...", 0). */
+PMOD_EXPORT int SNPRINTF(char *buf, size_t size, const char *fmt, ...)
+{
+  int res;
+  va_list args;
+  va_start (args, fmt);
+  res = VSNPRINTF (buf, size, fmt, args);
+  va_end (args);
+  return res;
+}
+#endif
+
 #ifndef HAVE_VFPRINTF
-PMOD_EXPORT int VFPRINTF(FILE *f,char *s,va_list args)
+PMOD_EXPORT int VFPRINTF(FILE *f,const char *s,va_list args)
 {
-  int i;
   char buffer[10000];
-  VSPRINTF(buffer,s,args);
-  i=strlen(buffer);
-  if(i+1>sizeof(buffer))
-    fatal("Buffer overflow.\n");
+  VSNPRINTF(buffer,sizeof(buffer),s,args);
   return fwrite(buffer,i,1,f);
 }
 #endif
@@ -659,67 +748,300 @@ PMOD_EXPORT INT32 EXTRACT_INT_(unsigned char *p)
 }
 #endif
 
+#ifdef HAVE_BROKEN_CHKSTK
+/* Intels ecl compiler adds calls to _stkchk() in the prologue,
+ * MicroSoft seems to have renamed it to __stkchk() in later
+ * versions of their SDK. This kludge attempts to workaround
+ * that compiler bug.
+ *	/grubba 2001-02-17.
+ */
+extern size_t __chkstk();
+size_t _chkstk() { return __chkstk(); }
+#endif /* HAVE_BROKEN_CHKSTK */
+
 #ifdef OWN_GETHRTIME
 
 #ifdef OWN_GETHRTIME_RDTSC
 
+/* Estimated errors in max standard deviation in seconds between the
+ * realtime and rtsc clocks. */
+
+#define GETTIMEOFDAY_ERR 1e-3
+/* Conservative value for gettimeofday resolution: It's taken from a
+ * Linux with SMP kernel where it's reliable only down to msec. */
+
+#define CPUINFO_ERR 1e-4
+/* Only count on four accurate digits in the MHz figure in
+ * /proc/cpuinfo. */
+
+#define SATISFACTORY_ERR 1e-6
+/* This is a compromise between final accurancy and the risk that the
+ * system clock changes during calibration so that the zero time gets
+ * wrong. (It's still better than the typical gethrtime accurancy on
+ * Solaris.) */
+
+#define COUNT_THRESHOLD 10
+/* Number of measurements before we can trust the variance
+ * calculation. */
+
 static long long hrtime_rtsc_zero;
 static long long hrtime_rtsc_last;
+#ifdef PIKE_DEBUG
+static long long hrtime_max = 0;
+#endif
 static struct timeval hrtime_timeval_zero;
+static long long hrtime_rtsc_base, hrtime_nsec_base = 0;
 static long double hrtime_conv=0.0;
+static double hrtime_conv_var = GETTIMEOFDAY_ERR * GETTIMEOFDAY_ERR;
+static int hrtime_is_calibrated = 0;
 
-#define RTSC(x)   							\
-   __asm__ __volatile__ (  "rdtsc"                			\
-			   :"=a" (((unsigned long*)&x)[0]), 		\
-			   "=d" (((unsigned long*)&x)[1]))
+/*  #define RTSC_DEBUG 1 */
+/*  #define RTSC_DEBUG_MORE 1 */
+
+#define RTSC(l,h)							\
+   __asm__ __volatile__ (  "rdtsc"					\
+			   :"=a" (l),					\
+			   "=d" (h))
+
+static INLINE long long rtsc()
+{
+   long long now;
+   unsigned long nl,nh;
+   RTSC(nl,nh);
+   return (((long long)nh)<<32)|nl;
+}
 
 void own_gethrtime_init()
 {
-   RTSC(hrtime_rtsc_zero);
-   hrtime_rtsc_last=hrtime_rtsc_zero;
+   int fd;
+
    GETTIMEOFDAY(&hrtime_timeval_zero);
+   hrtime_rtsc_zero=rtsc();
+   hrtime_rtsc_last = hrtime_rtsc_base = hrtime_rtsc_zero;
+#ifdef RTSC_DEBUG
+   fprintf(stderr,"init: %lld\n",hrtime_rtsc_zero);
+#endif   
+
+   do {
+     fd = fd_open("/proc/cpuinfo", fd_RDONLY, 0);
+   } while (fd < 0 && errno == EINTR);
+   if (fd >= 0) {
+     char buf[10240];
+     ptrdiff_t len;
+
+     set_nonblocking(fd, 1);	/* Paranoia. */
+     do {
+       len = fd_read(fd, buf, sizeof(buf) - 1);
+     } while (len < 0 && errno == EINTR);
+     fd_close(fd);
+
+     if (len > 0) {
+       char *p;
+       buf[len] = 0;
+       p = STRSTR(buf, "\ncpu MHz");
+       if (p) {
+	 p += sizeof("\ncpu MHz");
+	 p += STRCSPN(p, "0123456789\n");
+	 if (*p != '\n') {
+	   long long hz = 0;
+
+	   for (; *p >= '0' && *p <= '9'; p++)
+	     hz = hz * 10 + (*p - '0') * 1000000;
+	   if (*p == '.') {
+	     int mult = 100000;
+	     for (p++; mult && *p >= '0' && *p <= '9'; p++, mult /= 10)
+	       hz += (*p - '0') * mult;
+
+	     hrtime_conv = 1e9 / (long double) hz;
+	     hrtime_conv_var = CPUINFO_ERR * CPUINFO_ERR;
+#ifdef RTSC_DEBUG
+	     fprintf(stderr, "cpu Mhz read from /proc/cpuinfo: %lld.%lld, "
+		     "conv=%#.10Lg, est err=%#.3g\n",
+		     hz / 1000000, hz % 1000000, hrtime_conv, sqrt(hrtime_conv_var));
+#endif
+
+	     if (hrtime_conv_var < GETTIMEOFDAY_ERR * GETTIMEOFDAY_ERR) {
+#ifdef RTSC_DEBUG
+	       fprintf(stderr, "using rtsc from the start\n");
+#endif
+	       hrtime_is_calibrated = 1;
+	     }
+	   }
+	 }
+       }
+     }
+   }
 }
 
 void own_gethrtime_update(struct timeval *ptr)
 {
-   long long td,t,now;
-   RTSC(now);
+   long long td,t;
+   long long now;
+   long double conv;
+   double var;
+   static long long td_last = 0;
+   static long long w_sum = 0.0, w_c_sum = 0.0;
+   static double w_c_c_sum = 0.0;
+   static int count = -COUNT_THRESHOLD;
+
    GETTIMEOFDAY(ptr);
+   now=rtsc();
+
+#ifdef RTSC_DEBUG
+   fprintf(stderr,"update: rtsc now: %lld\n",now);
+#endif
+
+   if (hrtime_is_calibrated == 2) return; /* Calibration done. */
+
    td=((long long)ptr->tv_sec-hrtime_timeval_zero.tv_sec)*1000000000+
       ((long long)ptr->tv_usec-hrtime_timeval_zero.tv_usec)*1000;
-   hrtime_rtsc_last=now;
+
+   if (!hrtime_is_calibrated) {
+     /* We're in a hurry to make the rtsc kick in - require only
+      * hundreds of seconds between measurements. */
+     if (td - td_last < 10000000) return;
+     /* Drop the count threshold if a second has gone by. That ought
+      * to be long enough to get a reasonable reading. */
+     if (td - td_last > 1000000000) count = 0;
+   }
+   else
+     /* Doing fine calibration - require an order of whole seconds
+      * between measurements. */
+     if (now - hrtime_rtsc_last < 1000000000) return;
+
    t=now-hrtime_rtsc_zero;
-   if (t) hrtime_conv=((long double)td)/t;
+   if (t <= 0 || td <= 0)
+     /* Ouch, someone must have backed the clock(s). This situation
+      * could be handled better. */
+     return;
+   td_last = td;
+   hrtime_rtsc_last=now;
+   conv=((long double)td)/t;
 
 /* fixme: add time deviation detection;
    when time is adjusted, this calculation isn't linear anymore,
    so it might be best to reset it */
 
-#if 0
-   fprintf(stderr,"conv=%.8llg MHz=%.8llg\n",hrtime_conv,1000.0/hrtime_conv);
+   /* Calculate the variance. Each conversion factor is weighted by
+    * the time it was measured over, since later factors can be
+    * considered more exact due to the elapsed time. */
+   w_sum += t;
+   w_c_sum += td;		/* = t * conv */
+   w_c_c_sum += td * conv;	/* = t * conv * conv */
+
+   if (++count >= 0) {
+     double var = (w_c_c_sum - (double) w_c_sum * w_c_sum / w_sum) / w_sum;
+
+     if (var > 0.0 && var < hrtime_conv_var) {
+       hrtime_nsec_base +=
+	 (long long) ( (long double)(now-hrtime_rtsc_base) * hrtime_conv );
+       hrtime_rtsc_base = now;
+       hrtime_conv_var = var;
+       hrtime_conv = (long double) w_c_sum / w_sum;
+
+#ifdef RTSC_DEBUG
+       fprintf(stderr, "[%d, %#5.4g] "
+	       "last=%#.10Lg, conv=%#.10Lg +/- %#.3g (%#.10Lg MHz)\n",
+	       count, td / 1e9,
+	       conv, hrtime_conv, sqrt(hrtime_conv_var), 1000.0/hrtime_conv);
 #endif
+
+       if (!hrtime_is_calibrated) {
+	 if (hrtime_conv_var < GETTIMEOFDAY_ERR * GETTIMEOFDAY_ERR) {
+#ifdef RTSC_DEBUG
+	   fprintf(stderr, "switching to rtsc after %g sec\n", td / 1e9);
+#endif
+	   hrtime_is_calibrated = 1;
+	   /* Reinitialize the statistics since it's hard to weight
+	    * away the initial burst of noisy measurements we might
+	    * have done to make rtsc good enough to use. */
+	   count = -COUNT_THRESHOLD;
+	   w_c_sum = w_sum = 0;
+	   w_c_c_sum = 0.0;
+	   /* The variance might be too low if we dropped the count
+	    * threshold above. */
+	   hrtime_conv_var = GETTIMEOFDAY_ERR * GETTIMEOFDAY_ERR;
+	 }
+       }
+       else
+	 if (hrtime_conv_var < SATISFACTORY_ERR * SATISFACTORY_ERR) {
+#ifdef RTSC_DEBUG
+	   fprintf(stderr, "rtsc calibration finished, took %g sec\n", td / 1e9);
+#endif
+	   hrtime_is_calibrated = 2;
+	 }
+     }
+     else {
+#ifdef RTSC_DEBUG
+       fprintf(stderr, "[%d, %#5.4g] last=%#.10Lg +/- %#.3g, conv=%#.10Lg\n",
+	       count, td / 1e9, conv, var > 0.0 ? sqrt(var) : -1.0, hrtime_conv);
+#endif
+     }
+   }
+   else {
+#ifdef RTSC_DEBUG
+     fprintf(stderr, "[%d, %#5.4g] last=%#.10Lg\n", count, td / 1e9, conv);
+#endif
+   }
 }
 
 long long gethrtime()
 {
    long long now;
-   struct timeval dummy;
+   struct timeval tv;
 
-   if (hrtime_conv==0.0) own_gethrtime_update(&dummy);
-
-   RTSC(now);
-
-/* 2 seconds between updates */
-   if (now-hrtime_rtsc_last > 2000000000) 
-   {
-/*      fprintf(stderr,"update: %.8llg\n",1e-9*(now-hrtime_rtsc_last)); */
-      own_gethrtime_update(&dummy);
-      return gethrtime();
+   if (!hrtime_is_calibrated) {
+     own_gethrtime_update(&tv);
+     if (hrtime_is_calibrated) goto use_rtsc;
+     hrtime_rtsc_base=rtsc();
+     now=hrtime_nsec_base =
+       ((long long)tv.tv_sec-hrtime_timeval_zero.tv_sec)*1000000000+
+       ((long long)tv.tv_usec-hrtime_timeval_zero.tv_usec)*1000;
+   }
+   else {
+     now=rtsc();
+     if ((hrtime_is_calibrated == 1) &
+	 (now - hrtime_rtsc_last > 1000000000)) { /* Some seconds between updates. */
+       own_gethrtime_update(&tv);
+     use_rtsc:
+       now=rtsc();
+     }
+#ifdef RTSC_DEBUG_MORE
+     fprintf(stderr,"rtime: rtsc now: %lld\n",now);
+#endif
+     now = (long long) ( (long double)(now-hrtime_rtsc_base) * hrtime_conv ) +
+       hrtime_nsec_base;
    }
 
-   return (long long) ( (long double)now * hrtime_conv );
+#ifdef RTSC_DEBUG_MORE
+   fprintf(stderr,"(%lld)\n",now);
+#endif
+
+#ifdef PIKE_DEBUG
+   if (now<hrtime_max)
+     Pike_fatal("Time calculation went backwards with %lld.\n", hrtime_max - now);
+   hrtime_max = now;
+#endif
+   return now;
 }
 
+#endif	/* OWN_GETHRTIME_RDTSC */
+
+#endif	/* OWN_GETHRTIME */
+
+#ifndef HAVE_LDEXP
+double LDEXP(double x, int exp)
+{
+  return x * pow(2.0,(double)exp);
+}
 #endif
 
+#ifndef HAVE_FREXP
+double FREXP(double x, int *exp)
+{
+  double ret;
+  *exp = DO_NOT_WARN((int)ceil(log(fabs(x))/log(2.0)));
+  ret = (x*pow(2.0,(double)-*exp));
+  return ret;
+}
 #endif
