@@ -2,7 +2,7 @@
 || This file is part of Pike. For copyright information see COPYRIGHT.
 || Pike is distributed under GPL, LGPL and MPL. See the file COPYING
 || for more information.
-|| $Id: svalue.c,v 1.163 2005/09/09 15:09:08 mast Exp $
+|| $Id: svalue.c,v 1.193 2004/04/17 23:22:50 mast Exp $
 */
 
 #include "global.h"
@@ -25,54 +25,12 @@
 #include "queue.h"
 #include "bignum.h"
 #include "cyclic.h"
+#include "pike_float.h"
 #include <math.h>
-#ifdef HAVE_FLOATINGPOINT_H
-#include <floatingpoint.h>
-#endif
-
-#ifdef HAVE_IEEEFP_H
-#include <ieeefp.h>
-#endif
-
-#ifdef HAVE_FLOAT_H
-#include <float.h>
-#endif /* HAVE_FLOAT_H */
 
 #define sp Pike_sp
 
-/* isnan()...
- */
-#ifdef HAVE_ISNAN
-#if defined(HAVE__ISNAN) && defined(__NT__)
-/* On NT only _isnan() has a prototype.
- * isnan() is the standardized name, so use that
- * on all other platforms.
- */
-#define PIKE_ISNAN(X)	_isnan(X)
-#else /* !(HAVE__ISNAN && __NT__) */
-#define PIKE_ISNAN(X)	isnan(X)
-#endif /* HAVE__ISNAN && __NT__ */
-#else /* !HAVE_ISNAN */
-#ifdef HAVE__ISNAN
-#define PIKE_ISNAN(X)	_isnan(X)
-#else /* !HAVE__ISNAN */
-/* Fallback function */
-static int pike_isnan(double x)
-{
-  return ((x == 0.0) == (x < 0.0)) &&
-    ((x == 0.0) == (x > 0.0));
-}
-#define PIKE_ISNAN(X)	pike_isnan(X)
-#endif /* HAVE__ISNAN */
-#endif /* HAVE_ISNAN */
-
-#ifdef HAVE_ISUNORDERED
-#define PIKE_ISUNORDERED(X,Y) isunordered(X,Y)
-#else
-#define PIKE_ISUNORDERED(X,Y) (PIKE_ISNAN(X)||PIKE_ISNAN(Y))
-#endif /* HAVE_ISUNORDERED */
-
-RCSID("$Id: svalue.c,v 1.163 2005/09/09 15:09:08 mast Exp $");
+RCSID("$Id: svalue.c,v 1.193 2004/04/17 23:22:50 mast Exp $");
 
 struct svalue dest_ob_zero = {
   T_INT, 0,
@@ -283,7 +241,7 @@ PMOD_EXPORT void debug_free_svalues(struct svalue *s, size_t num, INT32 type_hin
 #ifdef DEBUG_MALLOC
       debug_malloc_update_location(s->u.refs  DMALLOC_PROXY_ARGS);
 #endif
-      if(--s->u.refs[0]<=0)
+      if(!sub_ref(s->u.dummy))
       {
 	really_free_svalue(s);
 	DO_IF_DMALLOC(s->u.refs=0);
@@ -299,7 +257,7 @@ PMOD_EXPORT void debug_free_svalues(struct svalue *s, size_t num, INT32 type_hin
 #ifdef DEBUG_MALLOC
       debug_malloc_update_location(s->u.refs  DMALLOC_PROXY_ARGS);
 #endif
-      if(--s->u.refs[0] <= 0)
+      if(!sub_ref(s->u.dummy))
       {
 	if(s->subtype == FUNCTION_BUILTIN)
 	  really_free_callable(s->u.efun);
@@ -338,53 +296,47 @@ PMOD_EXPORT void debug_free_mixed_svalues(struct svalue *s, size_t num, INT32 ty
 }
 
 
-PMOD_EXPORT void assign_svalues_no_free(struct svalue *to,
-			    const struct svalue *from,
-			    size_t num,
-			    TYPE_FIELD type_hint)
+PMOD_EXPORT TYPE_FIELD assign_svalues_no_free(struct svalue *to,
+					      const struct svalue *from,
+					      size_t num,
+					      TYPE_FIELD type_hint)
 {
   TYPE_FIELD masked_type;
 
-#ifdef PIKE_DEBUG
-  if(d_flag)
-  {
-    size_t e;
-    for(e=0;e<num;e++)
-      if(!(type_hint & (1<<from[e].type)))
-	 Pike_fatal("Type hint lies (%ld %ld %d)!\n",
-	       DO_NOT_WARN((long)e),
-	       (long)type_hint, from[e].type);
-  }
-#endif
+  check_type_hint (from, num, type_hint);
   MEMCPY((char *)to, (char *)from, sizeof(struct svalue) * num);
 
-  if (!(masked_type = (type_hint & ((2<<MAX_REF_TYPE)-1))))
-    return;
+  if (!(masked_type = (type_hint & BIT_REF_TYPES)))
+    return type_hint;
 
   if(masked_type == type_hint)
   {
     while(num--) {
-      add_ref(from->u.array);
+      add_ref(from->u.dummy);
       from++;
     }
-    return;
+    return type_hint;
   }
 
+  type_hint = 0;
   while(num--) {
+    type_hint |= 1 << from->type;
     if (from->type <= MAX_REF_TYPE) {
-      add_ref(from->u.array);
+      add_ref(from->u.dummy);
     }
     from++;
   }
+  return type_hint;
 }
 
-PMOD_EXPORT void assign_svalues(struct svalue *to,
-		    const struct svalue *from,
-		    size_t num,
-		    TYPE_FIELD types)
+PMOD_EXPORT TYPE_FIELD assign_svalues(struct svalue *to,
+				      const struct svalue *from,
+				      size_t num,
+				      TYPE_FIELD type_hint)
 {
-  free_mixed_svalues(to,num);
-  assign_svalues_no_free(to,from,num,types);
+  check_type_hint (from, num, type_hint);
+  if (type_hint & BIT_REF_TYPES) free_mixed_svalues(to,num);
+  return assign_svalues_no_free(to,from,num,type_hint);
 }
 
 PMOD_EXPORT void assign_to_short_svalue(union anything *u,
@@ -396,18 +348,17 @@ PMOD_EXPORT void assign_to_short_svalue(union anything *u,
 
   if(s->type == type)
   {
-    INT32 *tmp;
     switch(type)
     {
       case T_INT: u->integer=s->u.integer; break;
       case T_FLOAT: u->float_number=s->u.float_number; break;
       default:
-	if(u->refs && --*(u->refs) <= 0) really_free_short_svalue(u,type);
-	u->refs = tmp = s->u.refs;
-	tmp[0]++;
+	if(u->refs && !sub_ref(u->dummy)) really_free_short_svalue(u,type);
+	u->refs = s->u.refs;
+	add_ref(u->dummy);
     }
   }else if(type<=MAX_REF_TYPE && UNSAFE_IS_ZERO(s)){
-    if(u->refs && --*(u->refs) <= 0) really_free_short_svalue(u,type);
+    if(u->refs && !sub_ref(u->dummy)) really_free_short_svalue(u,type);
     u->refs=0;
   }else{
     Pike_error("Wrong type in assignment, expected %s, got %s.\n",
@@ -425,14 +376,13 @@ PMOD_EXPORT void assign_to_short_svalue_no_free(union anything *u,
 
   if(s->type == type)
   {
-    INT32 *tmp;
     switch(type)
     {
       case T_INT: u->integer=s->u.integer; break;
       case T_FLOAT: u->float_number=s->u.float_number; break;
       default:
-	u->refs = tmp = s->u.refs;
-	tmp[0]++;
+	u->refs = s->u.refs;
+	add_ref(u->dummy);
     }
   }else if(type<=MAX_REF_TYPE && UNSAFE_IS_ZERO(s)){
     u->refs=0;
@@ -501,7 +451,7 @@ PMOD_EXPORT void assign_short_svalue(union anything *to,
     case T_INT: to->integer=from->integer; break;
     case T_FLOAT: to->float_number=from->float_number; break;
     default:
-      if(to->refs && --*(to->refs) <= 0) really_free_short_svalue(to,type);
+      if(to->refs && !sub_ref(to->dummy)) really_free_short_svalue(to,type);
       to->refs = tmp = from->refs;
       if(tmp) tmp[0]++;
   }
@@ -535,11 +485,12 @@ PMOD_EXPORT unsigned INT32 hash_svalue(const struct svalue *s)
       pop_stack();
       break;
     }
+    /* FALL THROUGH */
   default:
 #if SIZEOF_CHAR_P > 4
-    q=DO_NOT_WARN((unsigned INT32)((ptrdiff_t)s->u.refs >> 2));
+    q=DO_NOT_WARN((unsigned INT32)(PTR_TO_INT(s->u.refs) >> 2));
 #else
-    q=DO_NOT_WARN((unsigned INT32)((ptrdiff_t)s->u.refs));
+    q=DO_NOT_WARN((unsigned INT32)(PTR_TO_INT(s->u.refs)));
 #endif
     break;
   case T_INT:   q=s->u.integer; break;
@@ -582,8 +533,7 @@ PMOD_EXPORT int svalue_is_true(const struct svalue *s)
 #if 0
       /* We should never get a function svalue for a prototype. */
       struct identifier *i = ID_FROM_INT(s->u.object->prog, s->subtype);
-      if (((i->identifier_flags & (IDENTIFIER_FUNCTION|IDENTIFIER_CONSTANT)) ==
-	   IDENTIFIER_PIKE_FUNCTION) &&
+      if (IDENTIFIER_IS_PIKE_FUNCTION(i->identifier_flags) &&
 	  (i->func.offset == -1)) {
 	/* Prototype. */
 	return 0;
@@ -641,8 +591,7 @@ PMOD_EXPORT int safe_svalue_is_true(const struct svalue *s)
 #if 0
       /* We should never get a function svalue for a prototype. */
       struct identifier *i = ID_FROM_INT(s->u.object->prog, s->subtype);
-      if (((i->identifier_flags & (IDENTIFIER_FUNCTION|IDENTIFIER_CONSTANT)) ==
-	   IDENTIFIER_PIKE_FUNCTION) &&
+      if (IDENTIFIER_IS_PIKE_FUNCTION(i->identifier_flags) &&
 	  (i->func.offset == -1)) {
 	/* Prototype. */
 	return 0;
@@ -737,8 +686,7 @@ PMOD_EXPORT int is_eq(const struct svalue *a, const struct svalue *b)
       if(FIND_LFUN(a->u.object->prog,LFUN_EQ) != -1)
       {
       a_is_obj:
-	assign_svalue_no_free(sp, b);
-	sp++;
+	push_svalue(b);
 	apply_lfun(a->u.object, LFUN_EQ, 1);
 	if(UNSAFE_IS_ZERO(sp-1))
 	{
@@ -762,8 +710,7 @@ PMOD_EXPORT int is_eq(const struct svalue *a, const struct svalue *b)
       if(FIND_LFUN(b->u.object->prog,LFUN_EQ) != -1)
       {
       b_is_obj:
-	assign_svalue_no_free(sp, a);
-	sp++;
+	push_svalue(a);
 	apply_lfun(b->u.object, LFUN_EQ, 1);
 	if(UNSAFE_IS_ZERO(sp-1))
 	{
@@ -781,7 +728,6 @@ PMOD_EXPORT int is_eq(const struct svalue *a, const struct svalue *b)
   switch(a->type)
   {
   case T_OBJECT:
-    if (a->u.object == b->u.object) return 1;
     if(FIND_LFUN(a->u.object->prog,LFUN_EQ) != -1)
       goto a_is_obj;
 
@@ -831,7 +777,9 @@ PMOD_EXPORT int is_eq(const struct svalue *a, const struct svalue *b)
     return a->u.float_number == b->u.float_number;
 
   default:
+#ifdef PIKE_DEBUG
     Pike_fatal("Unknown type %x\n",a->type);
+#endif
     return 0; /* make gcc happy */
   }
 }
@@ -965,35 +913,41 @@ PMOD_EXPORT int is_lt(const struct svalue *a, const struct svalue *b)
 
   if (a->type != b->type)
   {
-    if(a->type == T_FLOAT && b->type==T_INT)
+    int a_is_obj_without_lt;
+
+    if(a->type == T_FLOAT && b->type==T_INT) {
 #ifdef HAVE_ISLESS
       return isless(a->u.float_number, (FLOAT_TYPE)b->u.integer);
 #else
       return a->u.float_number < (FLOAT_TYPE)b->u.integer;
 #endif
+    }
 
-    if(a->type == T_INT && b->type==T_FLOAT)
+    if(a->type == T_INT && b->type==T_FLOAT) {
 #ifdef HAVE_ISLESS
       return isless((FLOAT_TYPE)a->u.integer, b->u.float_number);
 #else
       return (FLOAT_TYPE)a->u.integer < b->u.float_number;
 #endif
+    }
 
-  if (((a->type == T_TYPE) ||
-       (a->type == T_FUNCTION) || (a->type == T_PROGRAM)) &&
-      ((b->type == T_FUNCTION) ||
-       (b->type == T_PROGRAM) || (b->type == T_TYPE)))
-    goto compare_types;
+    if (((a->type == T_TYPE) ||
+	 (a->type == T_FUNCTION) || (a->type == T_PROGRAM)) &&
+	((b->type == T_FUNCTION) ||
+	 (b->type == T_PROGRAM) || (b->type == T_TYPE)))
+      goto compare_types;
 
     if(a->type == T_OBJECT)
     {
     a_is_object:
+#if 0
+      /* safe_check_destructed should avoid this. */
       if(!a->u.object->prog)
 	Pike_error("Comparison on destructed object.\n");
+#endif
       if(FIND_LFUN(a->u.object->prog,LFUN_LT) != -1)
       {
-	assign_svalue_no_free(sp, b);
-	sp++;
+	push_svalue(b);
 	apply_lfun(a->u.object, LFUN_LT, 1);
 	if(UNSAFE_IS_ZERO(sp-1))
 	{
@@ -1008,17 +962,30 @@ PMOD_EXPORT int is_lt(const struct svalue *a, const struct svalue *b)
 	  pop_stack();
 	  return 1;
 	}
+	a_is_obj_without_lt = 0;
       }
+      else
+	a_is_obj_without_lt = 1;
     }
+    else
+      a_is_obj_without_lt = 0;
 
     if(b->type == T_OBJECT)
     {
+#if 0
+      /* safe_check_destructed should avoid this. */
       if(!b->u.object->prog)
 	Pike_error("Comparison on destructed object.\n");
-      if(FIND_LFUN(b->u.object->prog,LFUN_GT) == -1)
-	Pike_error("Object lacks `>\n");
-      assign_svalue_no_free(sp, a);
-      sp++;
+#endif
+      if(FIND_LFUN(b->u.object->prog,LFUN_GT) == -1) {
+	if (a_is_obj_without_lt)
+	  Pike_error ("Object a lacks `< and object b lacks `> "
+		      "in comparison on the form a < b.\n");
+	else
+	  Pike_error ("Object b lacks `> "
+		      "in comparison on the form a < b.\n");
+      }
+      push_svalue(a);
       apply_lfun(b->u.object, LFUN_GT, 1);
       if(UNSAFE_IS_ZERO(sp-1))
       {
@@ -1034,22 +1001,28 @@ PMOD_EXPORT int is_lt(const struct svalue *a, const struct svalue *b)
 	return 1;
       }
     }
-    
-    Pike_error("Cannot compare different types.\n");
+
+    if (a_is_obj_without_lt)
+      Pike_error ("Object a lacks `< "
+		  "in comparison on the form a < b.\n");
+    else
+      Pike_error ("Cannot compare different types.\n");
   }
+
   switch(a->type)
   {
     case T_OBJECT:
       goto a_is_object;
       
     default:
-      Pike_error("Bad type to comparison.\n");
+      Pike_error("Cannot compare values of type %s.\n",
+		 get_name_of_type (a->type));
       
     case T_INT:
       return a->u.integer < b->u.integer;
       
     case T_STRING:
-      return my_strcmp(a->u.string, b->u.string) < 0;
+      return my_quick_strcmp(a->u.string, b->u.string) < 0;
       
     case T_FLOAT:
 #ifdef HAVE_ISLESS
@@ -1070,7 +1043,7 @@ PMOD_EXPORT int is_lt(const struct svalue *a, const struct svalue *b)
       int res;
       aa.u.program = program_from_svalue(a);
       if (!aa.u.program) {
-	Pike_error("Bad argument to comparison.");
+	Pike_error("Bad argument to comparison.\n");
       }
       type_stack_mark();
       push_object_type(0, aa.u.program->id);
@@ -1086,7 +1059,7 @@ PMOD_EXPORT int is_lt(const struct svalue *a, const struct svalue *b)
       int res;
       bb.u.program = program_from_svalue(b);
       if (!bb.u.program) {
-	Pike_error("Bad argument to comparison.");
+	Pike_error("Bad argument to comparison.\n");
       }
       type_stack_mark();
       push_object_type(0, bb.u.program->id);
@@ -1160,20 +1133,12 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 {
   char buf[50];
 
-  /* This needs to be a bit lower than LOW_C_STACK_MARGIN so that the
-   * the raw error can be printed in exit_on_error. */
-  check_c_stack(250);
-
   check_type(s->type);
   check_refs(s);
 
   indent+=2;
   switch(s->type)
   {
-    case T_LVALUE:
-      my_strcat("lvalue");
-      break;
-
     case T_INT:
       sprintf(buf,"%"PRINTPIKEINT"d",s->u.integer);
       my_strcat(buf);
@@ -1211,6 +1176,20 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	    my_putchar('r');
 	    break;
 
+	  case '\f':
+	    my_putchar('\\');
+	    my_putchar('f');
+	    break;
+
+	  case '\a':
+	    my_putchar('\\');
+	    my_putchar('a');
+	    break;
+
+	  case '\v':
+	    my_putchar('\\');
+	    my_putchar('v');
+	    break;
 
             case '"':
             case '\\':
@@ -1300,10 +1279,11 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	      /* We require some tricky coding to make this work
 	       * with tracing...
 	       */
-	      int save_t_flag=t_flag;
-	      dynbuf_string save_buffer=complex_free_buf();
-	    
-	      t_flag=0;
+	      int save_t_flag=Pike_interpreter.trace_level;
+	      dynamic_buffer save_buf;
+	      save_buffer (&save_buf);
+
+	      Pike_interpreter.trace_level=0;
 	      SET_CYCLIC_RET(1);
 	    
 	      ref_push_object(obj);
@@ -1319,8 +1299,8 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 		      push_text("(master returned illegal value from describe_module)");
 		    }
 	      
-		  init_buf_with_string(save_buffer);
-		  t_flag=save_t_flag;
+		  restore_buffer (&save_buf);
+		  Pike_interpreter.trace_level=save_t_flag;
 		
 		  dsv_add_string_to_buf( sp[-1].u.string );
 		  dsv_add_string_to_buf(name);
@@ -1329,11 +1309,9 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 		  END_CYCLIC();
 		  break;
 		}
-	      
-	      debug_malloc_touch(save_buffer.str);
-	      
-	      init_buf_with_string(save_buffer);
-	      t_flag=save_t_flag;
+
+	      restore_buffer (&save_buf);
+	      Pike_interpreter.trace_level=save_t_flag;
 	      pop_stack();
 	      prog = obj->prog;
 	    }
@@ -1341,8 +1319,7 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	  }
 
 	  if(name) {
-	    /* FIXME: Wide strings. */
-	    my_binary_strcat(name->str,name->len);
+	    dsv_add_string_to_buf(name);
 	    break;
 	  }
 	  else if (!prog) {
@@ -1350,12 +1327,13 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	    break;
 	  }
 	  else if (id && id->func.offset != -1) {
-	    struct pike_string *file;
+	    char *file;
 	    INT32 line;
-	    if ((file = low_get_line(prog->program + id->func.offset, prog, &line))) {
+	    if ((file = low_get_line_plain (prog->program + id->func.offset,
+					    prog, &line, 1))) {
 	      my_strcat("function(");
-	      my_strcat(file->str);
-	      free_string(file);
+	      my_strcat(file);
+	      free(file);
 	      if (line) {
 		sprintf(buf, ":%d", line);
 		my_strcat(buf);
@@ -1387,10 +1365,11 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	      /* We require some tricky coding to make this work
 	       * with tracing...
 	       */
-	      int save_t_flag=t_flag;
-	      dynbuf_string save_buffer=complex_free_buf();
+	      int save_t_flag=Pike_interpreter.trace_level;
+	      dynamic_buffer save_buf;
+	      save_buffer (&save_buf);
 	      
-	      t_flag=0;
+	      Pike_interpreter.trace_level=0;
 	      SET_CYCLIC_RET(1);
 	      
 	      debug_malloc_touch(obj);
@@ -1411,8 +1390,8 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 		      push_text("(object returned illegal value from _sprintf)");
 		    }
 
-		  init_buf_with_string(save_buffer);
-		  t_flag=save_t_flag;
+		  restore_buffer (&save_buf);
+		  Pike_interpreter.trace_level=save_t_flag;
 
 		  dsv_add_string_to_buf( sp[-1].u.string );
 
@@ -1421,10 +1400,8 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 		  break;
 		}
 
-	      debug_malloc_touch(save_buffer.str);
-
-	      init_buf_with_string(save_buffer);
-	      t_flag=save_t_flag;
+	      restore_buffer (&save_buf);
+	      Pike_interpreter.trace_level=save_t_flag;
 	      pop_stack();
 	      prog = obj->prog;
 	    }
@@ -1435,10 +1412,11 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	    /* We require some tricky coding to make this work
 	     * with tracing...
 	     */
-	    int save_t_flag=t_flag;
-	    dynbuf_string save_buffer=complex_free_buf();
-	    
-	    t_flag=0;
+	    int save_t_flag=Pike_interpreter.trace_level;
+	    dynamic_buffer save_buf;
+	    save_buffer (&save_buf);
+
+	    Pike_interpreter.trace_level=0;
 	    SET_CYCLIC_RET(1);
 	    
 	    debug_malloc_touch(obj);
@@ -1456,8 +1434,8 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 		    push_text("(master returned illegal value from describe_object)");
 		  }
 		
-		init_buf_with_string(save_buffer);
-		t_flag=save_t_flag;
+		restore_buffer (&save_buf);
+		Pike_interpreter.trace_level=save_t_flag;
 		
 		dsv_add_string_to_buf( sp[-1].u.string );
 
@@ -1465,11 +1443,9 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 		END_CYCLIC();
 		break;
 	      }
-	    
-	    debug_malloc_touch(save_buffer.str);
-	    
-	    init_buf_with_string(save_buffer);
-	    t_flag=save_t_flag;
+
+	    restore_buffer (&save_buf);
+	    Pike_interpreter.trace_level=save_t_flag;
 	    pop_stack();
 	    prog = obj->prog;
 	  }
@@ -1481,12 +1457,12 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	  break;
 	}
 	else {
-	  struct pike_string *file;
+	  char *file;
 	  INT32 line;
-	  if ((file = low_get_program_line(prog, &line))) {
+	  if ((file = low_get_program_line_plain (prog, &line, 1))) {
 	    my_strcat("object(");
-	    my_strcat(file->str);
-	    free_string(file);
+	    my_strcat(file);
+	    free(file);
 	    if (line) {
 	      sprintf(buf, ":%d", line);
 	      my_strcat(buf);
@@ -1512,10 +1488,11 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	  /* We require some tricky coding to make this work
 	   * with tracing...
 	   */
-	  int save_t_flag=t_flag;
-	  dynbuf_string save_buffer=complex_free_buf();
-	    
-	  t_flag=0;
+	  int save_t_flag=Pike_interpreter.trace_level;
+	  dynamic_buffer save_buf;
+	  save_buffer (&save_buf);
+
+	  Pike_interpreter.trace_level=0;
 	  SET_CYCLIC_RET(1);
 	    
 	  debug_malloc_touch(prog);
@@ -1533,8 +1510,8 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 		  push_text("(master returned illegal value from describe_program)");
 		}
 	      
-	      init_buf_with_string(save_buffer);
-	      t_flag=save_t_flag;
+	      restore_buffer (&save_buf);
+	      Pike_interpreter.trace_level=save_t_flag;
 		
 	      dsv_add_string_to_buf( sp[-1].u.string );
 
@@ -1542,23 +1519,21 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 	      END_CYCLIC();
 	      break;
 	    }
-	    
-	  debug_malloc_touch(save_buffer.str);
-	    
-	  init_buf_with_string(save_buffer);
-	  t_flag=save_t_flag;
+
+	  restore_buffer (&save_buf);
+	  Pike_interpreter.trace_level=save_t_flag;
 	  pop_stack();
 	}
 	END_CYCLIC();
       }
 
       {
-	struct pike_string *file;
+	char *file;
 	INT32 line;
-	if ((file = low_get_program_line(prog, &line))) {
+	if ((file = low_get_program_line_plain (prog, &line, 1))) {
 	  my_strcat("program(");
-	  my_strcat(file->str);
-	  free_string(file);
+	  my_strcat(file);
+	  free(file);
 	  if (line) {
 	    sprintf(buf, ":%d", line);
 	    my_strcat(buf);
@@ -1577,8 +1552,21 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
     }
 
     case T_FLOAT:
-      sprintf(buf,"%f",(double)s->u.float_number);
-      my_strcat(buf);
+      {
+	double d = s->u.float_number;
+	if (d != d) {
+	  my_strcat("nan");
+	} else if (d && (d+d == d)) {
+	  if (d > 0.0) {
+	    my_strcat("inf");
+	  } else {
+	    my_strcat("-inf");
+	  }
+	} else {
+	  sprintf(buf, "%f", d);
+	  my_strcat(buf);
+	}
+      }
       break;
 
     case T_ARRAY:
@@ -1601,17 +1589,27 @@ PMOD_EXPORT void describe_svalue(const struct svalue *s,int indent,struct proces
 
 PMOD_EXPORT void print_svalue (FILE *out, const struct svalue *s)
 {
-  dynbuf_string orig_str;
+  dynamic_buffer save_buf;
   dynbuf_string str;
-  orig_str = complex_free_buf();
-  init_buf();
+  init_buf(&save_buf);
   describe_svalue (s, 0, 0);
-  str = complex_free_buf();
-  if (orig_str.str) init_buf_with_string (orig_str);
+  str = complex_free_buf(&save_buf);
   fwrite (str.str, str.len, 1, out);
   free (str.str);
 }
 
+PMOD_EXPORT void print_short_svalue (FILE *out, const union anything *a, TYPE_T type)
+{
+  if (type <= MAX_REF_TYPE && !a->dummy)
+    fputs ("0", out);
+  else {
+    struct svalue sval;
+    sval.type = type;
+    sval.u = *a;
+    print_svalue (out, &sval);
+  }
+}
+
 PMOD_EXPORT void print_svalue_compact (FILE *out, const struct svalue *s)
 {
   switch (s->type) {
@@ -1626,9 +1624,11 @@ PMOD_EXPORT void print_svalue_compact (FILE *out, const struct svalue *s)
       break;
     case T_STRING:
       if (s->u.string->len > 80) {
-	push_string (string_slice (s->u.string, 0, 80));
-	print_svalue (out, Pike_sp - 1);
-	pop_stack();
+	struct svalue sval;
+	sval.type = T_STRING;
+	sval.u.string = string_slice (s->u.string, 0, 80);
+	print_svalue (out, &sval);
+	free_string (sval.u.string);
 	fprintf (out, "... (%d chars more)", s->u.string->len - 80);
 	break;
       }
@@ -1639,6 +1639,18 @@ PMOD_EXPORT void print_svalue_compact (FILE *out, const struct svalue *s)
   }
 }
 
+PMOD_EXPORT void print_short_svalue_compact (FILE *out, const union anything *a, TYPE_T type)
+{
+  if (type <= MAX_REF_TYPE && !a->dummy)
+    fputs ("0", out);
+  else {
+    struct svalue sval;
+    sval.type = type;
+    sval.u = *a;
+    print_svalue_compact (out, &sval);
+  }
+}
+
 PMOD_EXPORT void copy_svalues_recursively_no_free(struct svalue *to,
 				      const struct svalue *from,
 				      size_t num,
@@ -1677,10 +1689,63 @@ PMOD_EXPORT void copy_svalues_recursively_no_free(struct svalue *to,
 }
 
 #ifdef PIKE_DEBUG
+
+struct thorough_check_thing
+{
+  void *thing;
+  struct thorough_check_thing *next;
+};
+static struct thorough_check_thing *thoroughly_checking = NULL;
+
+void low_thorough_check_short_svalue (const union anything *u, TYPE_T type)
+{
+  struct thorough_check_thing checking;
+  struct thorough_check_thing *chk;
+  TYPE_T found_type;
+
+  if (type == T_STRING) {
+    if(!debug_findstring(u->string))
+      Pike_fatal("Shared string not shared!\n");
+    return;
+  }
+
+  /* Note: This fails if there are thread switches. */
+  checking.thing = u->ptr;
+  for (chk = thoroughly_checking; chk; chk = chk->next)
+    if (chk->thing == checking.thing)
+      return;
+  checking.next = thoroughly_checking;
+  thoroughly_checking = &checking;
+
+  found_type = attempt_to_identify (u->ptr, NULL);
+
+  if ((type == T_FUNCTION ? T_OBJECT : type) != found_type) {
+    if (found_type == PIKE_T_UNKNOWN && u->object->next == u->object)
+      {}			/* Ignore fake objects. */
+    else if (found_type == T_STRUCT_CALLABLE && type == T_FUNCTION)
+      {}			/* Built-in function. */
+    else {
+      describe (u->ptr);
+      Pike_fatal ("Thing at %p should be %s but is found to be %s.\n",
+		  u->ptr, get_name_of_type (type), get_name_of_type (found_type));
+    }
+  }
+
+  switch (type) {
+    case T_MAPPING: check_mapping(u->mapping); break;
+    case T_ARRAY: check_array(u->array); break;
+    case T_PROGRAM: check_program(u->program); break;
+    case T_OBJECT: check_object(u->object); break;
+    case T_MULTISET: check_multiset(u->multiset, 0); break;
+  }
+
+  if (thoroughly_checking != &checking)
+    Pike_fatal ("Non-thread assumption doesn't hold.\n");
+  thoroughly_checking = checking.next;
+}
+
 static void low_check_short_svalue(const union anything *u, TYPE_T type)
 {
-  static int inside=0;
-
   check_type(type);
   if ((type > MAX_REF_TYPE)||(!u->refs)) return;
 
@@ -1693,27 +1758,15 @@ static void low_check_short_svalue(const union anything *u, TYPE_T type)
 
   default:
     if(d_flag > 50)
-    {
-      if(inside) return;
-      inside=1;
-
-      switch(type)
-      {
-      case T_MAPPING: check_mapping(u->mapping); break;
-      case T_ARRAY: check_array(u->array); break;
-      case T_PROGRAM: check_program(u->program); break;
-      case T_OBJECT: check_object(u->object); break;
-/*      case T_MULTISET: check_multiset(u->multiset); break; */
-      }
-      inside=0;
-    }
+      low_thorough_check_short_svalue (u, type);
+    break;
   }
 }
 
 void check_short_svalue(const union anything *u, TYPE_T type)
 {
   if(type<=MAX_REF_TYPE &&
-     ((PIKE_INT32_ALIGNMENT-1) & (ptrdiff_t)(u->refs)))
+     ((PIKE_POINTER_ALIGNMENT-1) & (ptrdiff_t)(u->refs)))
     Pike_fatal("Odd pointer! type=%d u->refs=%p\n",type,u->refs);
 
   check_refs2(u,type);
@@ -1724,18 +1777,42 @@ void debug_check_svalue(const struct svalue *s)
 {
   check_type(s->type);
   if(s->type<=MAX_REF_TYPE &&
-     ((PIKE_INT32_ALIGNMENT-1) & (ptrdiff_t)(s->u.refs)))
+     ((PIKE_POINTER_ALIGNMENT-1) & (ptrdiff_t)(s->u.refs)))
     Pike_fatal("Odd pointer! type=%d u->refs=%p\n",s->type,s->u.refs);
 
+#if 0
+  if(s->type==T_INT) {
+    if(s->subtype!=NUMBER_NUMBER &&
+       s->subtype!=NUMBER_UNDEFINED && s->subtype!=NUMBER_DESTRUCTED) {
+      Pike_fatal("Unknown int subtype %d\n", s->subtype);
+    }
+  }
+  else if(s->type==T_FUNCTION) {
+    if(s->subtype!=0 && s->subtype!=FUNCTION_BUILTIN) {
+      Pike_fatal("Unknown function subtype %d\n", s->subtype);
+    }
+  }
+#endif
+
   check_refs(s);
   low_check_short_svalue(& s->u, s->type);
 }
 
-#endif
+void debug_check_type_hint (const struct svalue *svals, size_t num, TYPE_FIELD type_hint)
+{
+  if(d_flag)
+  {
+    size_t e;
+    for(e=0;e<num;e++)
+      if(!(type_hint & (1<<svals[e].type)))
+	Pike_fatal("Type hint lies (%"PRINTSIZET"d %ld %d)!\n",
+		   e, (long)type_hint, svals[e].type);
+  }
+}
 
-#ifdef PIKE_DEBUG
 /* NOTE: Must handle num being negative. */
-PMOD_EXPORT void real_gc_xmark_svalues(const struct svalue *s, ptrdiff_t num)
+PMOD_EXPORT void real_gc_mark_external_svalues(const struct svalue *s, ptrdiff_t num,
+					       const char *place)
 {
   ptrdiff_t e;
 
@@ -1750,11 +1827,12 @@ PMOD_EXPORT void real_gc_xmark_svalues(const struct svalue *s, ptrdiff_t num)
     gc_svalue_location=(void *)s;
 
     if(s->type <= MAX_REF_TYPE)
-      gc_external_mark2(s->u.refs,0,0);
+      gc_mark_external (s->u.refs, place);
   }
   gc_svalue_location=0;
 }
-#endif
+
+#endif	/* PIKE_DEBUG */
 
 
 /* Macro mania follows. We construct the gc 1) check, 2) mark, and 3)
@@ -1766,7 +1844,7 @@ PMOD_EXPORT void real_gc_xmark_svalues(const struct svalue *s, ptrdiff_t num)
     case T_FUNCTION:							\
       PRE DO_FUNC(U, T, ZAP, GC_DO)					\
     case T_OBJECT:							\
-      PRE DO_OBJ(U, T, ZAP, GC_DO)					\
+      PRE DO_OBJ(U, T, ZAP, GC_DO) break;				\
     case T_STRING:							\
       PRE DO_IF_DEBUG(if (d_flag) gc_check(U.string);) break;		\
     case T_PROGRAM:							\
@@ -1792,16 +1870,14 @@ PMOD_EXPORT void real_gc_xmark_svalues(const struct svalue *s, ptrdiff_t num)
       Pike_fatal("Cannot have a function in a short svalue.\n");
 
 #define DO_CHECK_OBJ(U, T, ZAP, GC_DO)					\
-      GC_DO(U.object);							\
-      break;
+      GC_DO(U.object);
 
 #define DO_CHECK_OBJ_WEAK(U, T, ZAP, GC_DO)				\
       if (U.object->prog &&						\
 	  !(U.object->prog->flags & PROGRAM_NO_WEAK_FREE))		\
 	gc_check_weak(U.object);					\
       else								\
-	gc_check(U.object);						\
-      break;
+	gc_check(U.object);
 
 #define NEVER_ZAP()
 
@@ -2109,7 +2185,8 @@ int gc_cycle_check_weak_short_svalue(union anything *u, TYPE_T type)
  * things in general during gc mark and cycle check passes, where
  * normal freeing is prohibited. If the thing runs out of refs, they
  * record them so that they're freed in the free pass along with the
- * rest.
+ * rest. If the gc isn't running, they behave just like free_svalue
+ * and free_short_svalue.
  *
  * Note that the gc will bug out if these are used on references that
  * have been accounted for by the recursing gc mark or cycle check
@@ -2117,25 +2194,29 @@ int gc_cycle_check_weak_short_svalue(union anything *u, TYPE_T type)
 
 void real_gc_free_svalue(struct svalue *s)
 {
+  if (Pike_in_gc && Pike_in_gc < GC_PASS_FREE) {
 #ifdef PIKE_DEBUG
-  if (Pike_in_gc != GC_PASS_MARK && Pike_in_gc != GC_PASS_CYCLE &&
-      Pike_in_gc != GC_PASS_ZAP_WEAK)
-    Pike_fatal("gc_free_svalue() called in invalid gc pass.\n");
+    if (Pike_in_gc != GC_PASS_MARK && Pike_in_gc != GC_PASS_CYCLE &&
+	Pike_in_gc != GC_PASS_ZAP_WEAK)
+      Pike_fatal("gc_free_svalue() called in invalid gc pass.\n");
 #endif
-  if (((1 << s->type) & BIT_COMPLEX) && *(s->u.refs) == 1)
-    gc_delayed_free(s->u.refs, s->type);
+    if (((1 << s->type) & BIT_COMPLEX) && *(s->u.refs) == 1)
+      gc_delayed_free(s->u.refs, s->type);
+  }
   free_svalue(s);
 }
 
 void real_gc_free_short_svalue(union anything *u, TYPE_T type)
 {
+  if (Pike_in_gc && Pike_in_gc < GC_PASS_FREE) {
 #ifdef PIKE_DEBUG
-  if (Pike_in_gc != GC_PASS_MARK && Pike_in_gc != GC_PASS_CYCLE &&
-      Pike_in_gc != GC_PASS_ZAP_WEAK)
-    Pike_fatal("gc_free_short_svalue() called in invalid gc pass.\n");
+    if (Pike_in_gc != GC_PASS_MARK && Pike_in_gc != GC_PASS_CYCLE &&
+	Pike_in_gc != GC_PASS_ZAP_WEAK)
+      Pike_fatal("gc_free_short_svalue() called in invalid gc pass.\n");
 #endif
-  if (((1 << type) & BIT_COMPLEX) && *u->refs == 1)
-    gc_delayed_free(u->refs, type);
+    if (((1 << type) & BIT_COMPLEX) && *u->refs == 1)
+      gc_delayed_free(u->refs, type);
+  }
   free_short_svalue(u, type);
 }
 
@@ -2158,6 +2239,7 @@ PMOD_EXPORT INT32 pike_sizeof(const struct svalue *s)
       apply_lfun(s->u.object, LFUN__SIZEOF, 0);
       if(sp[-1].type != T_INT)
 	Pike_error("Bad return type from o->_sizeof() (not int)\n");
+      dmalloc_touch_svalue(Pike_sp-1);
       sp--;
       return sp->u.integer;
     }
@@ -2172,6 +2254,7 @@ int svalues_are_constant(struct svalue *s,
 			 TYPE_FIELD hint,
 			 struct processing *p)
 {
+  check_type_hint (s, num, hint);
   if(hint & ~(BIT_STRING | BIT_INT | BIT_FLOAT))
   {
     INT32 e;
